#!/usr/bin/perl -w
######################################################################
#
# $Id: FTimes-EadRoutines.t,v 1.15 2019/03/14 16:07:42 klm Exp $
#
######################################################################
#
# Copyright 2008-2019 The FTimes Project, All Rights Reserved.
#
######################################################################
#
# Purpose: Tests for FTimes::EadRoutines.
#
######################################################################

use 5.008;
use strict;
use Test;

BEGIN
{
  my %hPlan =
  (
    'tests' => (1 + (6 * 256) + (2 * 768) + (2 * 1280)),
  );
  plan(%hPlan);
};

######################################################################
#
# Tests
#
######################################################################

  ####################################################################
  #
  # Test: The target module must load without error.
  #
  ####################################################################

  use FTimes::EadRoutines;
  ok(1);

  ####################################################################
  #
  # Common variables.
  #
  ####################################################################

  my (@aTuples, %hTargetValues);

  ####################################################################
  #
  # Test: URL encode 4-byte strings ranging from '%00' to '%ff'.
  #
  ####################################################################

  %hTargetValues =
  (
    '0x00' => '%00%00%00%00',
    '0x01' => '%01%01%01%01',
    '0x02' => '%02%02%02%02',
    '0x03' => '%03%03%03%03',
    '0x04' => '%04%04%04%04',
    '0x05' => '%05%05%05%05',
    '0x06' => '%06%06%06%06',
    '0x07' => '%07%07%07%07',
    '0x08' => '%08%08%08%08',
    '0x09' => '%09%09%09%09',
    '0x0a' => '%0a%0a%0a%0a',
    '0x0b' => '%0b%0b%0b%0b',
    '0x0c' => '%0c%0c%0c%0c',
    '0x0d' => '%0d%0d%0d%0d',
    '0x0e' => '%0e%0e%0e%0e',
    '0x0f' => '%0f%0f%0f%0f',
    '0x10' => '%10%10%10%10',
    '0x11' => '%11%11%11%11',
    '0x12' => '%12%12%12%12',
    '0x13' => '%13%13%13%13',
    '0x14' => '%14%14%14%14',
    '0x15' => '%15%15%15%15',
    '0x16' => '%16%16%16%16',
    '0x17' => '%17%17%17%17',
    '0x18' => '%18%18%18%18',
    '0x19' => '%19%19%19%19',
    '0x1a' => '%1a%1a%1a%1a',
    '0x1b' => '%1b%1b%1b%1b',
    '0x1c' => '%1c%1c%1c%1c',
    '0x1d' => '%1d%1d%1d%1d',
    '0x1e' => '%1e%1e%1e%1e',
    '0x1f' => '%1f%1f%1f%1f',
    '0x20' => '++++',
    '0x21' => '!!!!',
    '0x22' => '%22%22%22%22',
    '0x23' => '%23%23%23%23',
    '0x24' => '$$$$',
    '0x25' => '%25%25%25%25',
    '0x26' => '&&&&',
    '0x27' => '%27%27%27%27',
    '0x28' => '((((',
    '0x29' => '))))',
    '0x2a' => '****',
    '0x2b' => '%2b%2b%2b%2b',
    '0x2c' => ',,,,',
    '0x2d' => '----',
    '0x2e' => '....',
    '0x2f' => (($^O =~ /MSWin(32|64)/i) ? '%2f%2f%2f%2f' : '////'),
    '0x30' => '0000',
    '0x31' => '1111',
    '0x32' => '2222',
    '0x33' => '3333',
    '0x34' => '4444',
    '0x35' => '5555',
    '0x36' => '6666',
    '0x37' => '7777',
    '0x38' => '8888',
    '0x39' => '9999',
    '0x3a' => '::::',
    '0x3b' => ';;;;',
    '0x3c' => '<<<<',
    '0x3d' => '====',
    '0x3e' => '>>>>',
    '0x3f' => '????',
    '0x40' => '@@@@',
    '0x41' => 'AAAA',
    '0x42' => 'BBBB',
    '0x43' => 'CCCC',
    '0x44' => 'DDDD',
    '0x45' => 'EEEE',
    '0x46' => 'FFFF',
    '0x47' => 'GGGG',
    '0x48' => 'HHHH',
    '0x49' => 'IIII',
    '0x4a' => 'JJJJ',
    '0x4b' => 'KKKK',
    '0x4c' => 'LLLL',
    '0x4d' => 'MMMM',
    '0x4e' => 'NNNN',
    '0x4f' => 'OOOO',
    '0x50' => 'PPPP',
    '0x51' => 'QQQQ',
    '0x52' => 'RRRR',
    '0x53' => 'SSSS',
    '0x54' => 'TTTT',
    '0x55' => 'UUUU',
    '0x56' => 'VVVV',
    '0x57' => 'WWWW',
    '0x58' => 'XXXX',
    '0x59' => 'YYYY',
    '0x5a' => 'ZZZZ',
    '0x5b' => '[[[[',
    '0x5c' => (($^O =~ /MSWin(32|64)/i) ? '\\\\\\\\' : '%5c%5c%5c%5c'),
    '0x5d' => ']]]]',
    '0x5e' => '^^^^',
    '0x5f' => '____',
    '0x60' => '%60%60%60%60',
    '0x61' => 'aaaa',
    '0x62' => 'bbbb',
    '0x63' => 'cccc',
    '0x64' => 'dddd',
    '0x65' => 'eeee',
    '0x66' => 'ffff',
    '0x67' => 'gggg',
    '0x68' => 'hhhh',
    '0x69' => 'iiii',
    '0x6a' => 'jjjj',
    '0x6b' => 'kkkk',
    '0x6c' => 'llll',
    '0x6d' => 'mmmm',
    '0x6e' => 'nnnn',
    '0x6f' => 'oooo',
    '0x70' => 'pppp',
    '0x71' => 'qqqq',
    '0x72' => 'rrrr',
    '0x73' => 'ssss',
    '0x74' => 'tttt',
    '0x75' => 'uuuu',
    '0x76' => 'vvvv',
    '0x77' => 'wwww',
    '0x78' => 'xxxx',
    '0x79' => 'yyyy',
    '0x7a' => 'zzzz',
    '0x7b' => '{{{{',
    '0x7c' => '%7c%7c%7c%7c',
    '0x7d' => '}}}}',
    '0x7e' => '~~~~',
    '0x7f' => '%7f%7f%7f%7f',
    '0x80' => '%80%80%80%80',
    '0x81' => '%81%81%81%81',
    '0x82' => '%82%82%82%82',
    '0x83' => '%83%83%83%83',
    '0x84' => '%84%84%84%84',
    '0x85' => '%85%85%85%85',
    '0x86' => '%86%86%86%86',
    '0x87' => '%87%87%87%87',
    '0x88' => '%88%88%88%88',
    '0x89' => '%89%89%89%89',
    '0x8a' => '%8a%8a%8a%8a',
    '0x8b' => '%8b%8b%8b%8b',
    '0x8c' => '%8c%8c%8c%8c',
    '0x8d' => '%8d%8d%8d%8d',
    '0x8e' => '%8e%8e%8e%8e',
    '0x8f' => '%8f%8f%8f%8f',
    '0x90' => '%90%90%90%90',
    '0x91' => '%91%91%91%91',
    '0x92' => '%92%92%92%92',
    '0x93' => '%93%93%93%93',
    '0x94' => '%94%94%94%94',
    '0x95' => '%95%95%95%95',
    '0x96' => '%96%96%96%96',
    '0x97' => '%97%97%97%97',
    '0x98' => '%98%98%98%98',
    '0x99' => '%99%99%99%99',
    '0x9a' => '%9a%9a%9a%9a',
    '0x9b' => '%9b%9b%9b%9b',
    '0x9c' => '%9c%9c%9c%9c',
    '0x9d' => '%9d%9d%9d%9d',
    '0x9e' => '%9e%9e%9e%9e',
    '0x9f' => '%9f%9f%9f%9f',
    '0xa0' => '%a0%a0%a0%a0',
    '0xa1' => '%a1%a1%a1%a1',
    '0xa2' => '%a2%a2%a2%a2',
    '0xa3' => '%a3%a3%a3%a3',
    '0xa4' => '%a4%a4%a4%a4',
    '0xa5' => '%a5%a5%a5%a5',
    '0xa6' => '%a6%a6%a6%a6',
    '0xa7' => '%a7%a7%a7%a7',
    '0xa8' => '%a8%a8%a8%a8',
    '0xa9' => '%a9%a9%a9%a9',
    '0xaa' => '%aa%aa%aa%aa',
    '0xab' => '%ab%ab%ab%ab',
    '0xac' => '%ac%ac%ac%ac',
    '0xad' => '%ad%ad%ad%ad',
    '0xae' => '%ae%ae%ae%ae',
    '0xaf' => '%af%af%af%af',
    '0xb0' => '%b0%b0%b0%b0',
    '0xb1' => '%b1%b1%b1%b1',
    '0xb2' => '%b2%b2%b2%b2',
    '0xb3' => '%b3%b3%b3%b3',
    '0xb4' => '%b4%b4%b4%b4',
    '0xb5' => '%b5%b5%b5%b5',
    '0xb6' => '%b6%b6%b6%b6',
    '0xb7' => '%b7%b7%b7%b7',
    '0xb8' => '%b8%b8%b8%b8',
    '0xb9' => '%b9%b9%b9%b9',
    '0xba' => '%ba%ba%ba%ba',
    '0xbb' => '%bb%bb%bb%bb',
    '0xbc' => '%bc%bc%bc%bc',
    '0xbd' => '%bd%bd%bd%bd',
    '0xbe' => '%be%be%be%be',
    '0xbf' => '%bf%bf%bf%bf',
    '0xc0' => '%c0%c0%c0%c0',
    '0xc1' => '%c1%c1%c1%c1',
    '0xc2' => '%c2%c2%c2%c2',
    '0xc3' => '%c3%c3%c3%c3',
    '0xc4' => '%c4%c4%c4%c4',
    '0xc5' => '%c5%c5%c5%c5',
    '0xc6' => '%c6%c6%c6%c6',
    '0xc7' => '%c7%c7%c7%c7',
    '0xc8' => '%c8%c8%c8%c8',
    '0xc9' => '%c9%c9%c9%c9',
    '0xca' => '%ca%ca%ca%ca',
    '0xcb' => '%cb%cb%cb%cb',
    '0xcc' => '%cc%cc%cc%cc',
    '0xcd' => '%cd%cd%cd%cd',
    '0xce' => '%ce%ce%ce%ce',
    '0xcf' => '%cf%cf%cf%cf',
    '0xd0' => '%d0%d0%d0%d0',
    '0xd1' => '%d1%d1%d1%d1',
    '0xd2' => '%d2%d2%d2%d2',
    '0xd3' => '%d3%d3%d3%d3',
    '0xd4' => '%d4%d4%d4%d4',
    '0xd5' => '%d5%d5%d5%d5',
    '0xd6' => '%d6%d6%d6%d6',
    '0xd7' => '%d7%d7%d7%d7',
    '0xd8' => '%d8%d8%d8%d8',
    '0xd9' => '%d9%d9%d9%d9',
    '0xda' => '%da%da%da%da',
    '0xdb' => '%db%db%db%db',
    '0xdc' => '%dc%dc%dc%dc',
    '0xdd' => '%dd%dd%dd%dd',
    '0xde' => '%de%de%de%de',
    '0xdf' => '%df%df%df%df',
    '0xe0' => '%e0%e0%e0%e0',
    '0xe1' => '%e1%e1%e1%e1',
    '0xe2' => '%e2%e2%e2%e2',
    '0xe3' => '%e3%e3%e3%e3',
    '0xe4' => '%e4%e4%e4%e4',
    '0xe5' => '%e5%e5%e5%e5',
    '0xe6' => '%e6%e6%e6%e6',
    '0xe7' => '%e7%e7%e7%e7',
    '0xe8' => '%e8%e8%e8%e8',
    '0xe9' => '%e9%e9%e9%e9',
    '0xea' => '%ea%ea%ea%ea',
    '0xeb' => '%eb%eb%eb%eb',
    '0xec' => '%ec%ec%ec%ec',
    '0xed' => '%ed%ed%ed%ed',
    '0xee' => '%ee%ee%ee%ee',
    '0xef' => '%ef%ef%ef%ef',
    '0xf0' => '%f0%f0%f0%f0',
    '0xf1' => '%f1%f1%f1%f1',
    '0xf2' => '%f2%f2%f2%f2',
    '0xf3' => '%f3%f3%f3%f3',
    '0xf4' => '%f4%f4%f4%f4',
    '0xf5' => '%f5%f5%f5%f5',
    '0xf6' => '%f6%f6%f6%f6',
    '0xf7' => '%f7%f7%f7%f7',
    '0xf8' => '%f8%f8%f8%f8',
    '0xf9' => '%f9%f9%f9%f9',
    '0xfa' => '%fa%fa%fa%fa',
    '0xfb' => '%fb%fb%fb%fb',
    '0xfc' => '%fc%fc%fc%fc',
    '0xfd' => '%fd%fd%fd%fd',
    '0xfe' => '%fe%fe%fe%fe',
    '0xff' => '%ff%ff%ff%ff',
  );

  foreach my $sNumber (0..255)
  {
    my $sHexNumber = sprintf("0x%02x", $sNumber);
    my $sActualValue = EadFTimesUrlEncode(chr($sNumber) x 4);
    my $sTargetValue = $hTargetValues{$sHexNumber};
    ok($sActualValue eq $sTargetValue);
  }

  ####################################################################
  #
  # Test: URL decode 4-byte strings ranging from '%00' to '%ff'.
  #
  ####################################################################

  foreach my $sNumber (0..255)
  {
    my $sHexNumber = sprintf("0x%02x", $sNumber);
    my $sActualValue = EadFTimesUrlDecode($hTargetValues{$sHexNumber});
    my $sTargetValue = chr($sNumber) x 4;
    ok($sActualValue eq $sTargetValue);
  }

  ####################################################################
  #
  # Test: NSS encode 4-byte strings ranging from '%00' to '%ff'.
  #
  ####################################################################

  %hTargetValues =
  (
    '0x00' => '_00__00__00__00_',
    '0x01' => '_01__01__01__01_',
    '0x02' => '_02__02__02__02_',
    '0x03' => '_03__03__03__03_',
    '0x04' => '_04__04__04__04_',
    '0x05' => '_05__05__05__05_',
    '0x06' => '_06__06__06__06_',
    '0x07' => '_07__07__07__07_',
    '0x08' => '_08__08__08__08_',
    '0x09' => '_09__09__09__09_',
    '0x0a' => '_0a__0a__0a__0a_',
    '0x0b' => '_0b__0b__0b__0b_',
    '0x0c' => '_0c__0c__0c__0c_',
    '0x0d' => '_0d__0d__0d__0d_',
    '0x0e' => '_0e__0e__0e__0e_',
    '0x0f' => '_0f__0f__0f__0f_',
    '0x10' => '_10__10__10__10_',
    '0x11' => '_11__11__11__11_',
    '0x12' => '_12__12__12__12_',
    '0x13' => '_13__13__13__13_',
    '0x14' => '_14__14__14__14_',
    '0x15' => '_15__15__15__15_',
    '0x16' => '_16__16__16__16_',
    '0x17' => '_17__17__17__17_',
    '0x18' => '_18__18__18__18_',
    '0x19' => '_19__19__19__19_',
    '0x1a' => '_1a__1a__1a__1a_',
    '0x1b' => '_1b__1b__1b__1b_',
    '0x1c' => '_1c__1c__1c__1c_',
    '0x1d' => '_1d__1d__1d__1d_',
    '0x1e' => '_1e__1e__1e__1e_',
    '0x1f' => '_1f__1f__1f__1f_',
    '0x20' => '_20__20__20__20_',
    '0x21' => '_21__21__21__21_',
    '0x22' => '_22__22__22__22_',
    '0x23' => '_23__23__23__23_',
    '0x24' => '_24__24__24__24_',
    '0x25' => '_25__25__25__25_',
    '0x26' => '_26__26__26__26_',
    '0x27' => '_27__27__27__27_',
    '0x28' => '_28__28__28__28_',
    '0x29' => '_29__29__29__29_',
    '0x2a' => '_2a__2a__2a__2a_',
    '0x2b' => '_2b__2b__2b__2b_',
    '0x2c' => '_2c__2c__2c__2c_',
    '0x2d' => '----',
    '0x2e' => '....',
    '0x2f' => '////',
    '0x30' => '0000',
    '0x31' => '1111',
    '0x32' => '2222',
    '0x33' => '3333',
    '0x34' => '4444',
    '0x35' => '5555',
    '0x36' => '6666',
    '0x37' => '7777',
    '0x38' => '8888',
    '0x39' => '9999',
    '0x3a' => '_3a__3a__3a__3a_',
    '0x3b' => '_3b__3b__3b__3b_',
    '0x3c' => '_3c__3c__3c__3c_',
    '0x3d' => '_3d__3d__3d__3d_',
    '0x3e' => '_3e__3e__3e__3e_',
    '0x3f' => '_3f__3f__3f__3f_',
    '0x40' => '_40__40__40__40_',
    '0x41' => 'AAAA',
    '0x42' => 'BBBB',
    '0x43' => 'CCCC',
    '0x44' => 'DDDD',
    '0x45' => 'EEEE',
    '0x46' => 'FFFF',
    '0x47' => 'GGGG',
    '0x48' => 'HHHH',
    '0x49' => 'IIII',
    '0x4a' => 'JJJJ',
    '0x4b' => 'KKKK',
    '0x4c' => 'LLLL',
    '0x4d' => 'MMMM',
    '0x4e' => 'NNNN',
    '0x4f' => 'OOOO',
    '0x50' => 'PPPP',
    '0x51' => 'QQQQ',
    '0x52' => 'RRRR',
    '0x53' => 'SSSS',
    '0x54' => 'TTTT',
    '0x55' => 'UUUU',
    '0x56' => 'VVVV',
    '0x57' => 'WWWW',
    '0x58' => 'XXXX',
    '0x59' => 'YYYY',
    '0x5a' => 'ZZZZ',
    '0x5b' => '_5b__5b__5b__5b_',
    '0x5c' => '_5c__5c__5c__5c_',
    '0x5d' => '_5d__5d__5d__5d_',
    '0x5e' => '_5e__5e__5e__5e_',
    '0x5f' => '_5f__5f__5f__5f_',
    '0x60' => '_60__60__60__60_',
    '0x61' => 'aaaa',
    '0x62' => 'bbbb',
    '0x63' => 'cccc',
    '0x64' => 'dddd',
    '0x65' => 'eeee',
    '0x66' => 'ffff',
    '0x67' => 'gggg',
    '0x68' => 'hhhh',
    '0x69' => 'iiii',
    '0x6a' => 'jjjj',
    '0x6b' => 'kkkk',
    '0x6c' => 'llll',
    '0x6d' => 'mmmm',
    '0x6e' => 'nnnn',
    '0x6f' => 'oooo',
    '0x70' => 'pppp',
    '0x71' => 'qqqq',
    '0x72' => 'rrrr',
    '0x73' => 'ssss',
    '0x74' => 'tttt',
    '0x75' => 'uuuu',
    '0x76' => 'vvvv',
    '0x77' => 'wwww',
    '0x78' => 'xxxx',
    '0x79' => 'yyyy',
    '0x7a' => 'zzzz',
    '0x7b' => '_7b__7b__7b__7b_',
    '0x7c' => '_7c__7c__7c__7c_',
    '0x7d' => '_7d__7d__7d__7d_',
    '0x7e' => '_7e__7e__7e__7e_',
    '0x7f' => '_7f__7f__7f__7f_',
    '0x80' => '_80__80__80__80_',
    '0x81' => '_81__81__81__81_',
    '0x82' => '_82__82__82__82_',
    '0x83' => '_83__83__83__83_',
    '0x84' => '_84__84__84__84_',
    '0x85' => '_85__85__85__85_',
    '0x86' => '_86__86__86__86_',
    '0x87' => '_87__87__87__87_',
    '0x88' => '_88__88__88__88_',
    '0x89' => '_89__89__89__89_',
    '0x8a' => '_8a__8a__8a__8a_',
    '0x8b' => '_8b__8b__8b__8b_',
    '0x8c' => '_8c__8c__8c__8c_',
    '0x8d' => '_8d__8d__8d__8d_',
    '0x8e' => '_8e__8e__8e__8e_',
    '0x8f' => '_8f__8f__8f__8f_',
    '0x90' => '_90__90__90__90_',
    '0x91' => '_91__91__91__91_',
    '0x92' => '_92__92__92__92_',
    '0x93' => '_93__93__93__93_',
    '0x94' => '_94__94__94__94_',
    '0x95' => '_95__95__95__95_',
    '0x96' => '_96__96__96__96_',
    '0x97' => '_97__97__97__97_',
    '0x98' => '_98__98__98__98_',
    '0x99' => '_99__99__99__99_',
    '0x9a' => '_9a__9a__9a__9a_',
    '0x9b' => '_9b__9b__9b__9b_',
    '0x9c' => '_9c__9c__9c__9c_',
    '0x9d' => '_9d__9d__9d__9d_',
    '0x9e' => '_9e__9e__9e__9e_',
    '0x9f' => '_9f__9f__9f__9f_',
    '0xa0' => '_a0__a0__a0__a0_',
    '0xa1' => '_a1__a1__a1__a1_',
    '0xa2' => '_a2__a2__a2__a2_',
    '0xa3' => '_a3__a3__a3__a3_',
    '0xa4' => '_a4__a4__a4__a4_',
    '0xa5' => '_a5__a5__a5__a5_',
    '0xa6' => '_a6__a6__a6__a6_',
    '0xa7' => '_a7__a7__a7__a7_',
    '0xa8' => '_a8__a8__a8__a8_',
    '0xa9' => '_a9__a9__a9__a9_',
    '0xaa' => '_aa__aa__aa__aa_',
    '0xab' => '_ab__ab__ab__ab_',
    '0xac' => '_ac__ac__ac__ac_',
    '0xad' => '_ad__ad__ad__ad_',
    '0xae' => '_ae__ae__ae__ae_',
    '0xaf' => '_af__af__af__af_',
    '0xb0' => '_b0__b0__b0__b0_',
    '0xb1' => '_b1__b1__b1__b1_',
    '0xb2' => '_b2__b2__b2__b2_',
    '0xb3' => '_b3__b3__b3__b3_',
    '0xb4' => '_b4__b4__b4__b4_',
    '0xb5' => '_b5__b5__b5__b5_',
    '0xb6' => '_b6__b6__b6__b6_',
    '0xb7' => '_b7__b7__b7__b7_',
    '0xb8' => '_b8__b8__b8__b8_',
    '0xb9' => '_b9__b9__b9__b9_',
    '0xba' => '_ba__ba__ba__ba_',
    '0xbb' => '_bb__bb__bb__bb_',
    '0xbc' => '_bc__bc__bc__bc_',
    '0xbd' => '_bd__bd__bd__bd_',
    '0xbe' => '_be__be__be__be_',
    '0xbf' => '_bf__bf__bf__bf_',
    '0xc0' => '_c0__c0__c0__c0_',
    '0xc1' => '_c1__c1__c1__c1_',
    '0xc2' => '_c2__c2__c2__c2_',
    '0xc3' => '_c3__c3__c3__c3_',
    '0xc4' => '_c4__c4__c4__c4_',
    '0xc5' => '_c5__c5__c5__c5_',
    '0xc6' => '_c6__c6__c6__c6_',
    '0xc7' => '_c7__c7__c7__c7_',
    '0xc8' => '_c8__c8__c8__c8_',
    '0xc9' => '_c9__c9__c9__c9_',
    '0xca' => '_ca__ca__ca__ca_',
    '0xcb' => '_cb__cb__cb__cb_',
    '0xcc' => '_cc__cc__cc__cc_',
    '0xcd' => '_cd__cd__cd__cd_',
    '0xce' => '_ce__ce__ce__ce_',
    '0xcf' => '_cf__cf__cf__cf_',
    '0xd0' => '_d0__d0__d0__d0_',
    '0xd1' => '_d1__d1__d1__d1_',
    '0xd2' => '_d2__d2__d2__d2_',
    '0xd3' => '_d3__d3__d3__d3_',
    '0xd4' => '_d4__d4__d4__d4_',
    '0xd5' => '_d5__d5__d5__d5_',
    '0xd6' => '_d6__d6__d6__d6_',
    '0xd7' => '_d7__d7__d7__d7_',
    '0xd8' => '_d8__d8__d8__d8_',
    '0xd9' => '_d9__d9__d9__d9_',
    '0xda' => '_da__da__da__da_',
    '0xdb' => '_db__db__db__db_',
    '0xdc' => '_dc__dc__dc__dc_',
    '0xdd' => '_dd__dd__dd__dd_',
    '0xde' => '_de__de__de__de_',
    '0xdf' => '_df__df__df__df_',
    '0xe0' => '_e0__e0__e0__e0_',
    '0xe1' => '_e1__e1__e1__e1_',
    '0xe2' => '_e2__e2__e2__e2_',
    '0xe3' => '_e3__e3__e3__e3_',
    '0xe4' => '_e4__e4__e4__e4_',
    '0xe5' => '_e5__e5__e5__e5_',
    '0xe6' => '_e6__e6__e6__e6_',
    '0xe7' => '_e7__e7__e7__e7_',
    '0xe8' => '_e8__e8__e8__e8_',
    '0xe9' => '_e9__e9__e9__e9_',
    '0xea' => '_ea__ea__ea__ea_',
    '0xeb' => '_eb__eb__eb__eb_',
    '0xec' => '_ec__ec__ec__ec_',
    '0xed' => '_ed__ed__ed__ed_',
    '0xee' => '_ee__ee__ee__ee_',
    '0xef' => '_ef__ef__ef__ef_',
    '0xf0' => '_f0__f0__f0__f0_',
    '0xf1' => '_f1__f1__f1__f1_',
    '0xf2' => '_f2__f2__f2__f2_',
    '0xf3' => '_f3__f3__f3__f3_',
    '0xf4' => '_f4__f4__f4__f4_',
    '0xf5' => '_f5__f5__f5__f5_',
    '0xf6' => '_f6__f6__f6__f6_',
    '0xf7' => '_f7__f7__f7__f7_',
    '0xf8' => '_f8__f8__f8__f8_',
    '0xf9' => '_f9__f9__f9__f9_',
    '0xfa' => '_fa__fa__fa__fa_',
    '0xfb' => '_fb__fb__fb__fb_',
    '0xfc' => '_fc__fc__fc__fc_',
    '0xfd' => '_fd__fd__fd__fd_',
    '0xfe' => '_fe__fe__fe__fe_',
    '0xff' => '_ff__ff__ff__ff_',
  );

  foreach my $sNumber (0..255)
  {
    my $sHexNumber = sprintf("0x%02x", $sNumber);
    my $sActualValue = EadFTimesNssEncode(chr($sNumber) x 4);
    my $sTargetValue = $hTargetValues{$sHexNumber};
    ok($sActualValue eq $sTargetValue);
  }

  ####################################################################
  #
  # Test: URL decode 4-byte strings ranging from '%00' to '%ff'.
  #
  ####################################################################

  foreach my $sNumber (0..255)
  {
    my $sHexNumber = sprintf("0x%02x", $sNumber);
    my $sActualValue = EadFTimesNssDecode($hTargetValues{$sHexNumber});
    my $sTargetValue = chr($sNumber) x 4;
    ok($sActualValue eq $sTargetValue);
  }

  ####################################################################
  #
  # Test: SSS encode 4-byte strings ranging from '%00' to '%ff'.
  #
  ####################################################################

  %hTargetValues =
  (
    '0x00' => '%00%00%00%00',
    '0x01' => '%01%01%01%01',
    '0x02' => '%02%02%02%02',
    '0x03' => '%03%03%03%03',
    '0x04' => '%04%04%04%04',
    '0x05' => '%05%05%05%05',
    '0x06' => '%06%06%06%06',
    '0x07' => '%07%07%07%07',
    '0x08' => '%08%08%08%08',
    '0x09' => '%09%09%09%09',
    '0x0a' => '%0a%0a%0a%0a',
    '0x0b' => '%0b%0b%0b%0b',
    '0x0c' => '%0c%0c%0c%0c',
    '0x0d' => '%0d%0d%0d%0d',
    '0x0e' => '%0e%0e%0e%0e',
    '0x0f' => '%0f%0f%0f%0f',
    '0x10' => '%10%10%10%10',
    '0x11' => '%11%11%11%11',
    '0x12' => '%12%12%12%12',
    '0x13' => '%13%13%13%13',
    '0x14' => '%14%14%14%14',
    '0x15' => '%15%15%15%15',
    '0x16' => '%16%16%16%16',
    '0x17' => '%17%17%17%17',
    '0x18' => '%18%18%18%18',
    '0x19' => '%19%19%19%19',
    '0x1a' => '%1a%1a%1a%1a',
    '0x1b' => '%1b%1b%1b%1b',
    '0x1c' => '%1c%1c%1c%1c',
    '0x1d' => '%1d%1d%1d%1d',
    '0x1e' => '%1e%1e%1e%1e',
    '0x1f' => '%1f%1f%1f%1f',
    '0x20' => '++++',
    '0x21' => '%21%21%21%21',
    '0x22' => '%22%22%22%22',
    '0x23' => '%23%23%23%23',
    '0x24' => '%24%24%24%24',
    '0x25' => '%25%25%25%25',
    '0x26' => '%26%26%26%26',
    '0x27' => '%27%27%27%27',
    '0x28' => '%28%28%28%28',
    '0x29' => '%29%29%29%29',
    '0x2a' => '%2a%2a%2a%2a',
    '0x2b' => '%2b%2b%2b%2b',
    '0x2c' => ',,,,',
    '0x2d' => '----',
    '0x2e' => '....',
    '0x2f' => '////',
    '0x30' => '0000',
    '0x31' => '1111',
    '0x32' => '2222',
    '0x33' => '3333',
    '0x34' => '4444',
    '0x35' => '5555',
    '0x36' => '6666',
    '0x37' => '7777',
    '0x38' => '8888',
    '0x39' => '9999',
    '0x3a' => '::::',
    '0x3b' => '%3b%3b%3b%3b',
    '0x3c' => '%3c%3c%3c%3c',
    '0x3d' => '%3d%3d%3d%3d',
    '0x3e' => '%3e%3e%3e%3e',
    '0x3f' => '%3f%3f%3f%3f',
    '0x40' => '@@@@',
    '0x41' => 'AAAA',
    '0x42' => 'BBBB',
    '0x43' => 'CCCC',
    '0x44' => 'DDDD',
    '0x45' => 'EEEE',
    '0x46' => 'FFFF',
    '0x47' => 'GGGG',
    '0x48' => 'HHHH',
    '0x49' => 'IIII',
    '0x4a' => 'JJJJ',
    '0x4b' => 'KKKK',
    '0x4c' => 'LLLL',
    '0x4d' => 'MMMM',
    '0x4e' => 'NNNN',
    '0x4f' => 'OOOO',
    '0x50' => 'PPPP',
    '0x51' => 'QQQQ',
    '0x52' => 'RRRR',
    '0x53' => 'SSSS',
    '0x54' => 'TTTT',
    '0x55' => 'UUUU',
    '0x56' => 'VVVV',
    '0x57' => 'WWWW',
    '0x58' => 'XXXX',
    '0x59' => 'YYYY',
    '0x5a' => 'ZZZZ',
    '0x5b' => '%5b%5b%5b%5b',
    '0x5c' => '%5c%5c%5c%5c',
    '0x5d' => '%5d%5d%5d%5d',
    '0x5e' => '%5e%5e%5e%5e',
    '0x5f' => '____',
    '0x60' => '%60%60%60%60',
    '0x61' => 'aaaa',
    '0x62' => 'bbbb',
    '0x63' => 'cccc',
    '0x64' => 'dddd',
    '0x65' => 'eeee',
    '0x66' => 'ffff',
    '0x67' => 'gggg',
    '0x68' => 'hhhh',
    '0x69' => 'iiii',
    '0x6a' => 'jjjj',
    '0x6b' => 'kkkk',
    '0x6c' => 'llll',
    '0x6d' => 'mmmm',
    '0x6e' => 'nnnn',
    '0x6f' => 'oooo',
    '0x70' => 'pppp',
    '0x71' => 'qqqq',
    '0x72' => 'rrrr',
    '0x73' => 'ssss',
    '0x74' => 'tttt',
    '0x75' => 'uuuu',
    '0x76' => 'vvvv',
    '0x77' => 'wwww',
    '0x78' => 'xxxx',
    '0x79' => 'yyyy',
    '0x7a' => 'zzzz',
    '0x7b' => '%7b%7b%7b%7b',
    '0x7c' => '%7c%7c%7c%7c',
    '0x7d' => '%7d%7d%7d%7d',
    '0x7e' => '%7e%7e%7e%7e',
    '0x7f' => '%7f%7f%7f%7f',
    '0x80' => '%80%80%80%80',
    '0x81' => '%81%81%81%81',
    '0x82' => '%82%82%82%82',
    '0x83' => '%83%83%83%83',
    '0x84' => '%84%84%84%84',
    '0x85' => '%85%85%85%85',
    '0x86' => '%86%86%86%86',
    '0x87' => '%87%87%87%87',
    '0x88' => '%88%88%88%88',
    '0x89' => '%89%89%89%89',
    '0x8a' => '%8a%8a%8a%8a',
    '0x8b' => '%8b%8b%8b%8b',
    '0x8c' => '%8c%8c%8c%8c',
    '0x8d' => '%8d%8d%8d%8d',
    '0x8e' => '%8e%8e%8e%8e',
    '0x8f' => '%8f%8f%8f%8f',
    '0x90' => '%90%90%90%90',
    '0x91' => '%91%91%91%91',
    '0x92' => '%92%92%92%92',
    '0x93' => '%93%93%93%93',
    '0x94' => '%94%94%94%94',
    '0x95' => '%95%95%95%95',
    '0x96' => '%96%96%96%96',
    '0x97' => '%97%97%97%97',
    '0x98' => '%98%98%98%98',
    '0x99' => '%99%99%99%99',
    '0x9a' => '%9a%9a%9a%9a',
    '0x9b' => '%9b%9b%9b%9b',
    '0x9c' => '%9c%9c%9c%9c',
    '0x9d' => '%9d%9d%9d%9d',
    '0x9e' => '%9e%9e%9e%9e',
    '0x9f' => '%9f%9f%9f%9f',
    '0xa0' => '%a0%a0%a0%a0',
    '0xa1' => '%a1%a1%a1%a1',
    '0xa2' => '%a2%a2%a2%a2',
    '0xa3' => '%a3%a3%a3%a3',
    '0xa4' => '%a4%a4%a4%a4',
    '0xa5' => '%a5%a5%a5%a5',
    '0xa6' => '%a6%a6%a6%a6',
    '0xa7' => '%a7%a7%a7%a7',
    '0xa8' => '%a8%a8%a8%a8',
    '0xa9' => '%a9%a9%a9%a9',
    '0xaa' => '%aa%aa%aa%aa',
    '0xab' => '%ab%ab%ab%ab',
    '0xac' => '%ac%ac%ac%ac',
    '0xad' => '%ad%ad%ad%ad',
    '0xae' => '%ae%ae%ae%ae',
    '0xaf' => '%af%af%af%af',
    '0xb0' => '%b0%b0%b0%b0',
    '0xb1' => '%b1%b1%b1%b1',
    '0xb2' => '%b2%b2%b2%b2',
    '0xb3' => '%b3%b3%b3%b3',
    '0xb4' => '%b4%b4%b4%b4',
    '0xb5' => '%b5%b5%b5%b5',
    '0xb6' => '%b6%b6%b6%b6',
    '0xb7' => '%b7%b7%b7%b7',
    '0xb8' => '%b8%b8%b8%b8',
    '0xb9' => '%b9%b9%b9%b9',
    '0xba' => '%ba%ba%ba%ba',
    '0xbb' => '%bb%bb%bb%bb',
    '0xbc' => '%bc%bc%bc%bc',
    '0xbd' => '%bd%bd%bd%bd',
    '0xbe' => '%be%be%be%be',
    '0xbf' => '%bf%bf%bf%bf',
    '0xc0' => '%c0%c0%c0%c0',
    '0xc1' => '%c1%c1%c1%c1',
    '0xc2' => '%c2%c2%c2%c2',
    '0xc3' => '%c3%c3%c3%c3',
    '0xc4' => '%c4%c4%c4%c4',
    '0xc5' => '%c5%c5%c5%c5',
    '0xc6' => '%c6%c6%c6%c6',
    '0xc7' => '%c7%c7%c7%c7',
    '0xc8' => '%c8%c8%c8%c8',
    '0xc9' => '%c9%c9%c9%c9',
    '0xca' => '%ca%ca%ca%ca',
    '0xcb' => '%cb%cb%cb%cb',
    '0xcc' => '%cc%cc%cc%cc',
    '0xcd' => '%cd%cd%cd%cd',
    '0xce' => '%ce%ce%ce%ce',
    '0xcf' => '%cf%cf%cf%cf',
    '0xd0' => '%d0%d0%d0%d0',
    '0xd1' => '%d1%d1%d1%d1',
    '0xd2' => '%d2%d2%d2%d2',
    '0xd3' => '%d3%d3%d3%d3',
    '0xd4' => '%d4%d4%d4%d4',
    '0xd5' => '%d5%d5%d5%d5',
    '0xd6' => '%d6%d6%d6%d6',
    '0xd7' => '%d7%d7%d7%d7',
    '0xd8' => '%d8%d8%d8%d8',
    '0xd9' => '%d9%d9%d9%d9',
    '0xda' => '%da%da%da%da',
    '0xdb' => '%db%db%db%db',
    '0xdc' => '%dc%dc%dc%dc',
    '0xdd' => '%dd%dd%dd%dd',
    '0xde' => '%de%de%de%de',
    '0xdf' => '%df%df%df%df',
    '0xe0' => '%e0%e0%e0%e0',
    '0xe1' => '%e1%e1%e1%e1',
    '0xe2' => '%e2%e2%e2%e2',
    '0xe3' => '%e3%e3%e3%e3',
    '0xe4' => '%e4%e4%e4%e4',
    '0xe5' => '%e5%e5%e5%e5',
    '0xe6' => '%e6%e6%e6%e6',
    '0xe7' => '%e7%e7%e7%e7',
    '0xe8' => '%e8%e8%e8%e8',
    '0xe9' => '%e9%e9%e9%e9',
    '0xea' => '%ea%ea%ea%ea',
    '0xeb' => '%eb%eb%eb%eb',
    '0xec' => '%ec%ec%ec%ec',
    '0xed' => '%ed%ed%ed%ed',
    '0xee' => '%ee%ee%ee%ee',
    '0xef' => '%ef%ef%ef%ef',
    '0xf0' => '%f0%f0%f0%f0',
    '0xf1' => '%f1%f1%f1%f1',
    '0xf2' => '%f2%f2%f2%f2',
    '0xf3' => '%f3%f3%f3%f3',
    '0xf4' => '%f4%f4%f4%f4',
    '0xf5' => '%f5%f5%f5%f5',
    '0xf6' => '%f6%f6%f6%f6',
    '0xf7' => '%f7%f7%f7%f7',
    '0xf8' => '%f8%f8%f8%f8',
    '0xf9' => '%f9%f9%f9%f9',
    '0xfa' => '%fa%fa%fa%fa',
    '0xfb' => '%fb%fb%fb%fb',
    '0xfc' => '%fc%fc%fc%fc',
    '0xfd' => '%fd%fd%fd%fd',
    '0xfe' => '%fe%fe%fe%fe',
    '0xff' => '%ff%ff%ff%ff',
  );

  foreach my $sNumber (0..255)
  {
    my $sHexNumber = sprintf("0x%02x", $sNumber);
    my $sActualValue = EadFTimesSssEncode(chr($sNumber) x 4);
    my $sTargetValue = $hTargetValues{$sHexNumber};
    ok($sActualValue eq $sTargetValue);
  }

  ####################################################################
  #
  # Test: SSS decode 4-byte strings ranging from '%00' to '%ff'.
  #
  ####################################################################

  foreach my $sNumber (0..255)
  {
    my $sHexNumber = sprintf("0x%02x", $sNumber);
    my $sActualValue = EadFTimesSssDecode($hTargetValues{$sHexNumber});
    my $sTargetValue = chr($sNumber) x 4;
    ok($sActualValue eq $sTargetValue);
  }

  ####################################################################
  #
  # Test: Base64 encoding.
  #
  ####################################################################

  ####################################################################
  #
  # Note: The tuples array below was built using the following Python
  #       script:
  #
  # --- snip ---
  # #!/usr/bin/env python
  # import base64
  #
  # for i in range(256):
  #   for j in range(1, 4):
  #     a = []
  #     for k in range(j):
  #       a.append(i)
  #     v = bytearray(a)
  #     r = base64.b64encode(str(v));
  #     print "    [0x" + format(i, '02x') + ", " + str(j) + ", '" + r + "'],"
  # --- snip ---
  #
  ####################################################################

  @aTuples =
  (
    [0x00, 1, 'AA=='],
    [0x00, 2, 'AAA='],
    [0x00, 3, 'AAAA'],
    [0x01, 1, 'AQ=='],
    [0x01, 2, 'AQE='],
    [0x01, 3, 'AQEB'],
    [0x02, 1, 'Ag=='],
    [0x02, 2, 'AgI='],
    [0x02, 3, 'AgIC'],
    [0x03, 1, 'Aw=='],
    [0x03, 2, 'AwM='],
    [0x03, 3, 'AwMD'],
    [0x04, 1, 'BA=='],
    [0x04, 2, 'BAQ='],
    [0x04, 3, 'BAQE'],
    [0x05, 1, 'BQ=='],
    [0x05, 2, 'BQU='],
    [0x05, 3, 'BQUF'],
    [0x06, 1, 'Bg=='],
    [0x06, 2, 'BgY='],
    [0x06, 3, 'BgYG'],
    [0x07, 1, 'Bw=='],
    [0x07, 2, 'Bwc='],
    [0x07, 3, 'BwcH'],
    [0x08, 1, 'CA=='],
    [0x08, 2, 'CAg='],
    [0x08, 3, 'CAgI'],
    [0x09, 1, 'CQ=='],
    [0x09, 2, 'CQk='],
    [0x09, 3, 'CQkJ'],
    [0x0a, 1, 'Cg=='],
    [0x0a, 2, 'Cgo='],
    [0x0a, 3, 'CgoK'],
    [0x0b, 1, 'Cw=='],
    [0x0b, 2, 'Cws='],
    [0x0b, 3, 'CwsL'],
    [0x0c, 1, 'DA=='],
    [0x0c, 2, 'DAw='],
    [0x0c, 3, 'DAwM'],
    [0x0d, 1, 'DQ=='],
    [0x0d, 2, 'DQ0='],
    [0x0d, 3, 'DQ0N'],
    [0x0e, 1, 'Dg=='],
    [0x0e, 2, 'Dg4='],
    [0x0e, 3, 'Dg4O'],
    [0x0f, 1, 'Dw=='],
    [0x0f, 2, 'Dw8='],
    [0x0f, 3, 'Dw8P'],
    [0x10, 1, 'EA=='],
    [0x10, 2, 'EBA='],
    [0x10, 3, 'EBAQ'],
    [0x11, 1, 'EQ=='],
    [0x11, 2, 'ERE='],
    [0x11, 3, 'ERER'],
    [0x12, 1, 'Eg=='],
    [0x12, 2, 'EhI='],
    [0x12, 3, 'EhIS'],
    [0x13, 1, 'Ew=='],
    [0x13, 2, 'ExM='],
    [0x13, 3, 'ExMT'],
    [0x14, 1, 'FA=='],
    [0x14, 2, 'FBQ='],
    [0x14, 3, 'FBQU'],
    [0x15, 1, 'FQ=='],
    [0x15, 2, 'FRU='],
    [0x15, 3, 'FRUV'],
    [0x16, 1, 'Fg=='],
    [0x16, 2, 'FhY='],
    [0x16, 3, 'FhYW'],
    [0x17, 1, 'Fw=='],
    [0x17, 2, 'Fxc='],
    [0x17, 3, 'FxcX'],
    [0x18, 1, 'GA=='],
    [0x18, 2, 'GBg='],
    [0x18, 3, 'GBgY'],
    [0x19, 1, 'GQ=='],
    [0x19, 2, 'GRk='],
    [0x19, 3, 'GRkZ'],
    [0x1a, 1, 'Gg=='],
    [0x1a, 2, 'Gho='],
    [0x1a, 3, 'Ghoa'],
    [0x1b, 1, 'Gw=='],
    [0x1b, 2, 'Gxs='],
    [0x1b, 3, 'Gxsb'],
    [0x1c, 1, 'HA=='],
    [0x1c, 2, 'HBw='],
    [0x1c, 3, 'HBwc'],
    [0x1d, 1, 'HQ=='],
    [0x1d, 2, 'HR0='],
    [0x1d, 3, 'HR0d'],
    [0x1e, 1, 'Hg=='],
    [0x1e, 2, 'Hh4='],
    [0x1e, 3, 'Hh4e'],
    [0x1f, 1, 'Hw=='],
    [0x1f, 2, 'Hx8='],
    [0x1f, 3, 'Hx8f'],
    [0x20, 1, 'IA=='],
    [0x20, 2, 'ICA='],
    [0x20, 3, 'ICAg'],
    [0x21, 1, 'IQ=='],
    [0x21, 2, 'ISE='],
    [0x21, 3, 'ISEh'],
    [0x22, 1, 'Ig=='],
    [0x22, 2, 'IiI='],
    [0x22, 3, 'IiIi'],
    [0x23, 1, 'Iw=='],
    [0x23, 2, 'IyM='],
    [0x23, 3, 'IyMj'],
    [0x24, 1, 'JA=='],
    [0x24, 2, 'JCQ='],
    [0x24, 3, 'JCQk'],
    [0x25, 1, 'JQ=='],
    [0x25, 2, 'JSU='],
    [0x25, 3, 'JSUl'],
    [0x26, 1, 'Jg=='],
    [0x26, 2, 'JiY='],
    [0x26, 3, 'JiYm'],
    [0x27, 1, 'Jw=='],
    [0x27, 2, 'Jyc='],
    [0x27, 3, 'Jycn'],
    [0x28, 1, 'KA=='],
    [0x28, 2, 'KCg='],
    [0x28, 3, 'KCgo'],
    [0x29, 1, 'KQ=='],
    [0x29, 2, 'KSk='],
    [0x29, 3, 'KSkp'],
    [0x2a, 1, 'Kg=='],
    [0x2a, 2, 'Kio='],
    [0x2a, 3, 'Kioq'],
    [0x2b, 1, 'Kw=='],
    [0x2b, 2, 'Kys='],
    [0x2b, 3, 'Kysr'],
    [0x2c, 1, 'LA=='],
    [0x2c, 2, 'LCw='],
    [0x2c, 3, 'LCws'],
    [0x2d, 1, 'LQ=='],
    [0x2d, 2, 'LS0='],
    [0x2d, 3, 'LS0t'],
    [0x2e, 1, 'Lg=='],
    [0x2e, 2, 'Li4='],
    [0x2e, 3, 'Li4u'],
    [0x2f, 1, 'Lw=='],
    [0x2f, 2, 'Ly8='],
    [0x2f, 3, 'Ly8v'],
    [0x30, 1, 'MA=='],
    [0x30, 2, 'MDA='],
    [0x30, 3, 'MDAw'],
    [0x31, 1, 'MQ=='],
    [0x31, 2, 'MTE='],
    [0x31, 3, 'MTEx'],
    [0x32, 1, 'Mg=='],
    [0x32, 2, 'MjI='],
    [0x32, 3, 'MjIy'],
    [0x33, 1, 'Mw=='],
    [0x33, 2, 'MzM='],
    [0x33, 3, 'MzMz'],
    [0x34, 1, 'NA=='],
    [0x34, 2, 'NDQ='],
    [0x34, 3, 'NDQ0'],
    [0x35, 1, 'NQ=='],
    [0x35, 2, 'NTU='],
    [0x35, 3, 'NTU1'],
    [0x36, 1, 'Ng=='],
    [0x36, 2, 'NjY='],
    [0x36, 3, 'NjY2'],
    [0x37, 1, 'Nw=='],
    [0x37, 2, 'Nzc='],
    [0x37, 3, 'Nzc3'],
    [0x38, 1, 'OA=='],
    [0x38, 2, 'ODg='],
    [0x38, 3, 'ODg4'],
    [0x39, 1, 'OQ=='],
    [0x39, 2, 'OTk='],
    [0x39, 3, 'OTk5'],
    [0x3a, 1, 'Og=='],
    [0x3a, 2, 'Ojo='],
    [0x3a, 3, 'Ojo6'],
    [0x3b, 1, 'Ow=='],
    [0x3b, 2, 'Ozs='],
    [0x3b, 3, 'Ozs7'],
    [0x3c, 1, 'PA=='],
    [0x3c, 2, 'PDw='],
    [0x3c, 3, 'PDw8'],
    [0x3d, 1, 'PQ=='],
    [0x3d, 2, 'PT0='],
    [0x3d, 3, 'PT09'],
    [0x3e, 1, 'Pg=='],
    [0x3e, 2, 'Pj4='],
    [0x3e, 3, 'Pj4+'],
    [0x3f, 1, 'Pw=='],
    [0x3f, 2, 'Pz8='],
    [0x3f, 3, 'Pz8/'],
    [0x40, 1, 'QA=='],
    [0x40, 2, 'QEA='],
    [0x40, 3, 'QEBA'],
    [0x41, 1, 'QQ=='],
    [0x41, 2, 'QUE='],
    [0x41, 3, 'QUFB'],
    [0x42, 1, 'Qg=='],
    [0x42, 2, 'QkI='],
    [0x42, 3, 'QkJC'],
    [0x43, 1, 'Qw=='],
    [0x43, 2, 'Q0M='],
    [0x43, 3, 'Q0ND'],
    [0x44, 1, 'RA=='],
    [0x44, 2, 'REQ='],
    [0x44, 3, 'RERE'],
    [0x45, 1, 'RQ=='],
    [0x45, 2, 'RUU='],
    [0x45, 3, 'RUVF'],
    [0x46, 1, 'Rg=='],
    [0x46, 2, 'RkY='],
    [0x46, 3, 'RkZG'],
    [0x47, 1, 'Rw=='],
    [0x47, 2, 'R0c='],
    [0x47, 3, 'R0dH'],
    [0x48, 1, 'SA=='],
    [0x48, 2, 'SEg='],
    [0x48, 3, 'SEhI'],
    [0x49, 1, 'SQ=='],
    [0x49, 2, 'SUk='],
    [0x49, 3, 'SUlJ'],
    [0x4a, 1, 'Sg=='],
    [0x4a, 2, 'Sko='],
    [0x4a, 3, 'SkpK'],
    [0x4b, 1, 'Sw=='],
    [0x4b, 2, 'S0s='],
    [0x4b, 3, 'S0tL'],
    [0x4c, 1, 'TA=='],
    [0x4c, 2, 'TEw='],
    [0x4c, 3, 'TExM'],
    [0x4d, 1, 'TQ=='],
    [0x4d, 2, 'TU0='],
    [0x4d, 3, 'TU1N'],
    [0x4e, 1, 'Tg=='],
    [0x4e, 2, 'Tk4='],
    [0x4e, 3, 'Tk5O'],
    [0x4f, 1, 'Tw=='],
    [0x4f, 2, 'T08='],
    [0x4f, 3, 'T09P'],
    [0x50, 1, 'UA=='],
    [0x50, 2, 'UFA='],
    [0x50, 3, 'UFBQ'],
    [0x51, 1, 'UQ=='],
    [0x51, 2, 'UVE='],
    [0x51, 3, 'UVFR'],
    [0x52, 1, 'Ug=='],
    [0x52, 2, 'UlI='],
    [0x52, 3, 'UlJS'],
    [0x53, 1, 'Uw=='],
    [0x53, 2, 'U1M='],
    [0x53, 3, 'U1NT'],
    [0x54, 1, 'VA=='],
    [0x54, 2, 'VFQ='],
    [0x54, 3, 'VFRU'],
    [0x55, 1, 'VQ=='],
    [0x55, 2, 'VVU='],
    [0x55, 3, 'VVVV'],
    [0x56, 1, 'Vg=='],
    [0x56, 2, 'VlY='],
    [0x56, 3, 'VlZW'],
    [0x57, 1, 'Vw=='],
    [0x57, 2, 'V1c='],
    [0x57, 3, 'V1dX'],
    [0x58, 1, 'WA=='],
    [0x58, 2, 'WFg='],
    [0x58, 3, 'WFhY'],
    [0x59, 1, 'WQ=='],
    [0x59, 2, 'WVk='],
    [0x59, 3, 'WVlZ'],
    [0x5a, 1, 'Wg=='],
    [0x5a, 2, 'Wlo='],
    [0x5a, 3, 'Wlpa'],
    [0x5b, 1, 'Ww=='],
    [0x5b, 2, 'W1s='],
    [0x5b, 3, 'W1tb'],
    [0x5c, 1, 'XA=='],
    [0x5c, 2, 'XFw='],
    [0x5c, 3, 'XFxc'],
    [0x5d, 1, 'XQ=='],
    [0x5d, 2, 'XV0='],
    [0x5d, 3, 'XV1d'],
    [0x5e, 1, 'Xg=='],
    [0x5e, 2, 'Xl4='],
    [0x5e, 3, 'Xl5e'],
    [0x5f, 1, 'Xw=='],
    [0x5f, 2, 'X18='],
    [0x5f, 3, 'X19f'],
    [0x60, 1, 'YA=='],
    [0x60, 2, 'YGA='],
    [0x60, 3, 'YGBg'],
    [0x61, 1, 'YQ=='],
    [0x61, 2, 'YWE='],
    [0x61, 3, 'YWFh'],
    [0x62, 1, 'Yg=='],
    [0x62, 2, 'YmI='],
    [0x62, 3, 'YmJi'],
    [0x63, 1, 'Yw=='],
    [0x63, 2, 'Y2M='],
    [0x63, 3, 'Y2Nj'],
    [0x64, 1, 'ZA=='],
    [0x64, 2, 'ZGQ='],
    [0x64, 3, 'ZGRk'],
    [0x65, 1, 'ZQ=='],
    [0x65, 2, 'ZWU='],
    [0x65, 3, 'ZWVl'],
    [0x66, 1, 'Zg=='],
    [0x66, 2, 'ZmY='],
    [0x66, 3, 'ZmZm'],
    [0x67, 1, 'Zw=='],
    [0x67, 2, 'Z2c='],
    [0x67, 3, 'Z2dn'],
    [0x68, 1, 'aA=='],
    [0x68, 2, 'aGg='],
    [0x68, 3, 'aGho'],
    [0x69, 1, 'aQ=='],
    [0x69, 2, 'aWk='],
    [0x69, 3, 'aWlp'],
    [0x6a, 1, 'ag=='],
    [0x6a, 2, 'amo='],
    [0x6a, 3, 'ampq'],
    [0x6b, 1, 'aw=='],
    [0x6b, 2, 'a2s='],
    [0x6b, 3, 'a2tr'],
    [0x6c, 1, 'bA=='],
    [0x6c, 2, 'bGw='],
    [0x6c, 3, 'bGxs'],
    [0x6d, 1, 'bQ=='],
    [0x6d, 2, 'bW0='],
    [0x6d, 3, 'bW1t'],
    [0x6e, 1, 'bg=='],
    [0x6e, 2, 'bm4='],
    [0x6e, 3, 'bm5u'],
    [0x6f, 1, 'bw=='],
    [0x6f, 2, 'b28='],
    [0x6f, 3, 'b29v'],
    [0x70, 1, 'cA=='],
    [0x70, 2, 'cHA='],
    [0x70, 3, 'cHBw'],
    [0x71, 1, 'cQ=='],
    [0x71, 2, 'cXE='],
    [0x71, 3, 'cXFx'],
    [0x72, 1, 'cg=='],
    [0x72, 2, 'cnI='],
    [0x72, 3, 'cnJy'],
    [0x73, 1, 'cw=='],
    [0x73, 2, 'c3M='],
    [0x73, 3, 'c3Nz'],
    [0x74, 1, 'dA=='],
    [0x74, 2, 'dHQ='],
    [0x74, 3, 'dHR0'],
    [0x75, 1, 'dQ=='],
    [0x75, 2, 'dXU='],
    [0x75, 3, 'dXV1'],
    [0x76, 1, 'dg=='],
    [0x76, 2, 'dnY='],
    [0x76, 3, 'dnZ2'],
    [0x77, 1, 'dw=='],
    [0x77, 2, 'd3c='],
    [0x77, 3, 'd3d3'],
    [0x78, 1, 'eA=='],
    [0x78, 2, 'eHg='],
    [0x78, 3, 'eHh4'],
    [0x79, 1, 'eQ=='],
    [0x79, 2, 'eXk='],
    [0x79, 3, 'eXl5'],
    [0x7a, 1, 'eg=='],
    [0x7a, 2, 'eno='],
    [0x7a, 3, 'enp6'],
    [0x7b, 1, 'ew=='],
    [0x7b, 2, 'e3s='],
    [0x7b, 3, 'e3t7'],
    [0x7c, 1, 'fA=='],
    [0x7c, 2, 'fHw='],
    [0x7c, 3, 'fHx8'],
    [0x7d, 1, 'fQ=='],
    [0x7d, 2, 'fX0='],
    [0x7d, 3, 'fX19'],
    [0x7e, 1, 'fg=='],
    [0x7e, 2, 'fn4='],
    [0x7e, 3, 'fn5+'],
    [0x7f, 1, 'fw=='],
    [0x7f, 2, 'f38='],
    [0x7f, 3, 'f39/'],
    [0x80, 1, 'gA=='],
    [0x80, 2, 'gIA='],
    [0x80, 3, 'gICA'],
    [0x81, 1, 'gQ=='],
    [0x81, 2, 'gYE='],
    [0x81, 3, 'gYGB'],
    [0x82, 1, 'gg=='],
    [0x82, 2, 'goI='],
    [0x82, 3, 'goKC'],
    [0x83, 1, 'gw=='],
    [0x83, 2, 'g4M='],
    [0x83, 3, 'g4OD'],
    [0x84, 1, 'hA=='],
    [0x84, 2, 'hIQ='],
    [0x84, 3, 'hISE'],
    [0x85, 1, 'hQ=='],
    [0x85, 2, 'hYU='],
    [0x85, 3, 'hYWF'],
    [0x86, 1, 'hg=='],
    [0x86, 2, 'hoY='],
    [0x86, 3, 'hoaG'],
    [0x87, 1, 'hw=='],
    [0x87, 2, 'h4c='],
    [0x87, 3, 'h4eH'],
    [0x88, 1, 'iA=='],
    [0x88, 2, 'iIg='],
    [0x88, 3, 'iIiI'],
    [0x89, 1, 'iQ=='],
    [0x89, 2, 'iYk='],
    [0x89, 3, 'iYmJ'],
    [0x8a, 1, 'ig=='],
    [0x8a, 2, 'ioo='],
    [0x8a, 3, 'ioqK'],
    [0x8b, 1, 'iw=='],
    [0x8b, 2, 'i4s='],
    [0x8b, 3, 'i4uL'],
    [0x8c, 1, 'jA=='],
    [0x8c, 2, 'jIw='],
    [0x8c, 3, 'jIyM'],
    [0x8d, 1, 'jQ=='],
    [0x8d, 2, 'jY0='],
    [0x8d, 3, 'jY2N'],
    [0x8e, 1, 'jg=='],
    [0x8e, 2, 'jo4='],
    [0x8e, 3, 'jo6O'],
    [0x8f, 1, 'jw=='],
    [0x8f, 2, 'j48='],
    [0x8f, 3, 'j4+P'],
    [0x90, 1, 'kA=='],
    [0x90, 2, 'kJA='],
    [0x90, 3, 'kJCQ'],
    [0x91, 1, 'kQ=='],
    [0x91, 2, 'kZE='],
    [0x91, 3, 'kZGR'],
    [0x92, 1, 'kg=='],
    [0x92, 2, 'kpI='],
    [0x92, 3, 'kpKS'],
    [0x93, 1, 'kw=='],
    [0x93, 2, 'k5M='],
    [0x93, 3, 'k5OT'],
    [0x94, 1, 'lA=='],
    [0x94, 2, 'lJQ='],
    [0x94, 3, 'lJSU'],
    [0x95, 1, 'lQ=='],
    [0x95, 2, 'lZU='],
    [0x95, 3, 'lZWV'],
    [0x96, 1, 'lg=='],
    [0x96, 2, 'lpY='],
    [0x96, 3, 'lpaW'],
    [0x97, 1, 'lw=='],
    [0x97, 2, 'l5c='],
    [0x97, 3, 'l5eX'],
    [0x98, 1, 'mA=='],
    [0x98, 2, 'mJg='],
    [0x98, 3, 'mJiY'],
    [0x99, 1, 'mQ=='],
    [0x99, 2, 'mZk='],
    [0x99, 3, 'mZmZ'],
    [0x9a, 1, 'mg=='],
    [0x9a, 2, 'mpo='],
    [0x9a, 3, 'mpqa'],
    [0x9b, 1, 'mw=='],
    [0x9b, 2, 'm5s='],
    [0x9b, 3, 'm5ub'],
    [0x9c, 1, 'nA=='],
    [0x9c, 2, 'nJw='],
    [0x9c, 3, 'nJyc'],
    [0x9d, 1, 'nQ=='],
    [0x9d, 2, 'nZ0='],
    [0x9d, 3, 'nZ2d'],
    [0x9e, 1, 'ng=='],
    [0x9e, 2, 'np4='],
    [0x9e, 3, 'np6e'],
    [0x9f, 1, 'nw=='],
    [0x9f, 2, 'n58='],
    [0x9f, 3, 'n5+f'],
    [0xa0, 1, 'oA=='],
    [0xa0, 2, 'oKA='],
    [0xa0, 3, 'oKCg'],
    [0xa1, 1, 'oQ=='],
    [0xa1, 2, 'oaE='],
    [0xa1, 3, 'oaGh'],
    [0xa2, 1, 'og=='],
    [0xa2, 2, 'oqI='],
    [0xa2, 3, 'oqKi'],
    [0xa3, 1, 'ow=='],
    [0xa3, 2, 'o6M='],
    [0xa3, 3, 'o6Oj'],
    [0xa4, 1, 'pA=='],
    [0xa4, 2, 'pKQ='],
    [0xa4, 3, 'pKSk'],
    [0xa5, 1, 'pQ=='],
    [0xa5, 2, 'paU='],
    [0xa5, 3, 'paWl'],
    [0xa6, 1, 'pg=='],
    [0xa6, 2, 'pqY='],
    [0xa6, 3, 'pqam'],
    [0xa7, 1, 'pw=='],
    [0xa7, 2, 'p6c='],
    [0xa7, 3, 'p6en'],
    [0xa8, 1, 'qA=='],
    [0xa8, 2, 'qKg='],
    [0xa8, 3, 'qKio'],
    [0xa9, 1, 'qQ=='],
    [0xa9, 2, 'qak='],
    [0xa9, 3, 'qamp'],
    [0xaa, 1, 'qg=='],
    [0xaa, 2, 'qqo='],
    [0xaa, 3, 'qqqq'],
    [0xab, 1, 'qw=='],
    [0xab, 2, 'q6s='],
    [0xab, 3, 'q6ur'],
    [0xac, 1, 'rA=='],
    [0xac, 2, 'rKw='],
    [0xac, 3, 'rKys'],
    [0xad, 1, 'rQ=='],
    [0xad, 2, 'ra0='],
    [0xad, 3, 'ra2t'],
    [0xae, 1, 'rg=='],
    [0xae, 2, 'rq4='],
    [0xae, 3, 'rq6u'],
    [0xaf, 1, 'rw=='],
    [0xaf, 2, 'r68='],
    [0xaf, 3, 'r6+v'],
    [0xb0, 1, 'sA=='],
    [0xb0, 2, 'sLA='],
    [0xb0, 3, 'sLCw'],
    [0xb1, 1, 'sQ=='],
    [0xb1, 2, 'sbE='],
    [0xb1, 3, 'sbGx'],
    [0xb2, 1, 'sg=='],
    [0xb2, 2, 'srI='],
    [0xb2, 3, 'srKy'],
    [0xb3, 1, 'sw=='],
    [0xb3, 2, 's7M='],
    [0xb3, 3, 's7Oz'],
    [0xb4, 1, 'tA=='],
    [0xb4, 2, 'tLQ='],
    [0xb4, 3, 'tLS0'],
    [0xb5, 1, 'tQ=='],
    [0xb5, 2, 'tbU='],
    [0xb5, 3, 'tbW1'],
    [0xb6, 1, 'tg=='],
    [0xb6, 2, 'trY='],
    [0xb6, 3, 'tra2'],
    [0xb7, 1, 'tw=='],
    [0xb7, 2, 't7c='],
    [0xb7, 3, 't7e3'],
    [0xb8, 1, 'uA=='],
    [0xb8, 2, 'uLg='],
    [0xb8, 3, 'uLi4'],
    [0xb9, 1, 'uQ=='],
    [0xb9, 2, 'ubk='],
    [0xb9, 3, 'ubm5'],
    [0xba, 1, 'ug=='],
    [0xba, 2, 'uro='],
    [0xba, 3, 'urq6'],
    [0xbb, 1, 'uw=='],
    [0xbb, 2, 'u7s='],
    [0xbb, 3, 'u7u7'],
    [0xbc, 1, 'vA=='],
    [0xbc, 2, 'vLw='],
    [0xbc, 3, 'vLy8'],
    [0xbd, 1, 'vQ=='],
    [0xbd, 2, 'vb0='],
    [0xbd, 3, 'vb29'],
    [0xbe, 1, 'vg=='],
    [0xbe, 2, 'vr4='],
    [0xbe, 3, 'vr6+'],
    [0xbf, 1, 'vw=='],
    [0xbf, 2, 'v78='],
    [0xbf, 3, 'v7+/'],
    [0xc0, 1, 'wA=='],
    [0xc0, 2, 'wMA='],
    [0xc0, 3, 'wMDA'],
    [0xc1, 1, 'wQ=='],
    [0xc1, 2, 'wcE='],
    [0xc1, 3, 'wcHB'],
    [0xc2, 1, 'wg=='],
    [0xc2, 2, 'wsI='],
    [0xc2, 3, 'wsLC'],
    [0xc3, 1, 'ww=='],
    [0xc3, 2, 'w8M='],
    [0xc3, 3, 'w8PD'],
    [0xc4, 1, 'xA=='],
    [0xc4, 2, 'xMQ='],
    [0xc4, 3, 'xMTE'],
    [0xc5, 1, 'xQ=='],
    [0xc5, 2, 'xcU='],
    [0xc5, 3, 'xcXF'],
    [0xc6, 1, 'xg=='],
    [0xc6, 2, 'xsY='],
    [0xc6, 3, 'xsbG'],
    [0xc7, 1, 'xw=='],
    [0xc7, 2, 'x8c='],
    [0xc7, 3, 'x8fH'],
    [0xc8, 1, 'yA=='],
    [0xc8, 2, 'yMg='],
    [0xc8, 3, 'yMjI'],
    [0xc9, 1, 'yQ=='],
    [0xc9, 2, 'yck='],
    [0xc9, 3, 'ycnJ'],
    [0xca, 1, 'yg=='],
    [0xca, 2, 'yso='],
    [0xca, 3, 'ysrK'],
    [0xcb, 1, 'yw=='],
    [0xcb, 2, 'y8s='],
    [0xcb, 3, 'y8vL'],
    [0xcc, 1, 'zA=='],
    [0xcc, 2, 'zMw='],
    [0xcc, 3, 'zMzM'],
    [0xcd, 1, 'zQ=='],
    [0xcd, 2, 'zc0='],
    [0xcd, 3, 'zc3N'],
    [0xce, 1, 'zg=='],
    [0xce, 2, 'zs4='],
    [0xce, 3, 'zs7O'],
    [0xcf, 1, 'zw=='],
    [0xcf, 2, 'z88='],
    [0xcf, 3, 'z8/P'],
    [0xd0, 1, '0A=='],
    [0xd0, 2, '0NA='],
    [0xd0, 3, '0NDQ'],
    [0xd1, 1, '0Q=='],
    [0xd1, 2, '0dE='],
    [0xd1, 3, '0dHR'],
    [0xd2, 1, '0g=='],
    [0xd2, 2, '0tI='],
    [0xd2, 3, '0tLS'],
    [0xd3, 1, '0w=='],
    [0xd3, 2, '09M='],
    [0xd3, 3, '09PT'],
    [0xd4, 1, '1A=='],
    [0xd4, 2, '1NQ='],
    [0xd4, 3, '1NTU'],
    [0xd5, 1, '1Q=='],
    [0xd5, 2, '1dU='],
    [0xd5, 3, '1dXV'],
    [0xd6, 1, '1g=='],
    [0xd6, 2, '1tY='],
    [0xd6, 3, '1tbW'],
    [0xd7, 1, '1w=='],
    [0xd7, 2, '19c='],
    [0xd7, 3, '19fX'],
    [0xd8, 1, '2A=='],
    [0xd8, 2, '2Ng='],
    [0xd8, 3, '2NjY'],
    [0xd9, 1, '2Q=='],
    [0xd9, 2, '2dk='],
    [0xd9, 3, '2dnZ'],
    [0xda, 1, '2g=='],
    [0xda, 2, '2to='],
    [0xda, 3, '2tra'],
    [0xdb, 1, '2w=='],
    [0xdb, 2, '29s='],
    [0xdb, 3, '29vb'],
    [0xdc, 1, '3A=='],
    [0xdc, 2, '3Nw='],
    [0xdc, 3, '3Nzc'],
    [0xdd, 1, '3Q=='],
    [0xdd, 2, '3d0='],
    [0xdd, 3, '3d3d'],
    [0xde, 1, '3g=='],
    [0xde, 2, '3t4='],
    [0xde, 3, '3t7e'],
    [0xdf, 1, '3w=='],
    [0xdf, 2, '398='],
    [0xdf, 3, '39/f'],
    [0xe0, 1, '4A=='],
    [0xe0, 2, '4OA='],
    [0xe0, 3, '4ODg'],
    [0xe1, 1, '4Q=='],
    [0xe1, 2, '4eE='],
    [0xe1, 3, '4eHh'],
    [0xe2, 1, '4g=='],
    [0xe2, 2, '4uI='],
    [0xe2, 3, '4uLi'],
    [0xe3, 1, '4w=='],
    [0xe3, 2, '4+M='],
    [0xe3, 3, '4+Pj'],
    [0xe4, 1, '5A=='],
    [0xe4, 2, '5OQ='],
    [0xe4, 3, '5OTk'],
    [0xe5, 1, '5Q=='],
    [0xe5, 2, '5eU='],
    [0xe5, 3, '5eXl'],
    [0xe6, 1, '5g=='],
    [0xe6, 2, '5uY='],
    [0xe6, 3, '5ubm'],
    [0xe7, 1, '5w=='],
    [0xe7, 2, '5+c='],
    [0xe7, 3, '5+fn'],
    [0xe8, 1, '6A=='],
    [0xe8, 2, '6Og='],
    [0xe8, 3, '6Ojo'],
    [0xe9, 1, '6Q=='],
    [0xe9, 2, '6ek='],
    [0xe9, 3, '6enp'],
    [0xea, 1, '6g=='],
    [0xea, 2, '6uo='],
    [0xea, 3, '6urq'],
    [0xeb, 1, '6w=='],
    [0xeb, 2, '6+s='],
    [0xeb, 3, '6+vr'],
    [0xec, 1, '7A=='],
    [0xec, 2, '7Ow='],
    [0xec, 3, '7Ozs'],
    [0xed, 1, '7Q=='],
    [0xed, 2, '7e0='],
    [0xed, 3, '7e3t'],
    [0xee, 1, '7g=='],
    [0xee, 2, '7u4='],
    [0xee, 3, '7u7u'],
    [0xef, 1, '7w=='],
    [0xef, 2, '7+8='],
    [0xef, 3, '7+/v'],
    [0xf0, 1, '8A=='],
    [0xf0, 2, '8PA='],
    [0xf0, 3, '8PDw'],
    [0xf1, 1, '8Q=='],
    [0xf1, 2, '8fE='],
    [0xf1, 3, '8fHx'],
    [0xf2, 1, '8g=='],
    [0xf2, 2, '8vI='],
    [0xf2, 3, '8vLy'],
    [0xf3, 1, '8w=='],
    [0xf3, 2, '8/M='],
    [0xf3, 3, '8/Pz'],
    [0xf4, 1, '9A=='],
    [0xf4, 2, '9PQ='],
    [0xf4, 3, '9PT0'],
    [0xf5, 1, '9Q=='],
    [0xf5, 2, '9fU='],
    [0xf5, 3, '9fX1'],
    [0xf6, 1, '9g=='],
    [0xf6, 2, '9vY='],
    [0xf6, 3, '9vb2'],
    [0xf7, 1, '9w=='],
    [0xf7, 2, '9/c='],
    [0xf7, 3, '9/f3'],
    [0xf8, 1, '+A=='],
    [0xf8, 2, '+Pg='],
    [0xf8, 3, '+Pj4'],
    [0xf9, 1, '+Q=='],
    [0xf9, 2, '+fk='],
    [0xf9, 3, '+fn5'],
    [0xfa, 1, '+g=='],
    [0xfa, 2, '+vo='],
    [0xfa, 3, '+vr6'],
    [0xfb, 1, '+w=='],
    [0xfb, 2, '+/s='],
    [0xfb, 3, '+/v7'],
    [0xfc, 1, '/A=='],
    [0xfc, 2, '/Pw='],
    [0xfc, 3, '/Pz8'],
    [0xfd, 1, '/Q=='],
    [0xfd, 2, '/f0='],
    [0xfd, 3, '/f39'],
    [0xfe, 1, '/g=='],
    [0xfe, 2, '/v4='],
    [0xfe, 3, '/v7+'],
    [0xff, 1, '/w=='],
    [0xff, 2, '//8='],
    [0xff, 3, '////'],
  );

  foreach my $paTuple (@aTuples)
  {
    my $sString = pack("C", $$paTuple[0]) x $$paTuple[1];
    my $sActualValue = EadBase64Encode($sString);
    my $sTargetValue = $$paTuple[2];
    ok($sActualValue eq $sTargetValue);
  }

  ####################################################################
  #
  # Test: Base64 decoding.
  #
  ####################################################################

  foreach my $paTuple (@aTuples)
  {
    my $sHexNumber = sprintf("0x%02x", $$paTuple[0]);
    my $sActualValue = EadBase64Decode($$paTuple[2]);
    my $sTargetValue = chr($$paTuple[0]) x $$paTuple[1];
    ok($sActualValue eq $sTargetValue);
  }

  ####################################################################
  #
  # Test: Base32 encoding.
  #
  ####################################################################

  ####################################################################
  #
  # Note: The tuples array below was built using the following Python
  #       script:
  #
  # --- snip ---
  # #!/usr/bin/env python
  # import base64
  #
  # for i in range(256):
  #   for j in range(1, 6):
  #     a = []
  #     for k in range(j):
  #       a.append(i)
  #     v = bytearray(a)
  #     r = base64.b32encode(str(v));
  #     print "    [0x" + format(i, '02x') + ", " + str(j) + ", '" + r + "'],"
  # --- snip ---
  #
  ####################################################################

  @aTuples =
  (
    [0x00, 1, 'AA======'],
    [0x00, 2, 'AAAA===='],
    [0x00, 3, 'AAAAA==='],
    [0x00, 4, 'AAAAAAA='],
    [0x00, 5, 'AAAAAAAA'],
    [0x01, 1, 'AE======'],
    [0x01, 2, 'AEAQ===='],
    [0x01, 3, 'AEAQC==='],
    [0x01, 4, 'AEAQCAI='],
    [0x01, 5, 'AEAQCAIB'],
    [0x02, 1, 'AI======'],
    [0x02, 2, 'AIBA===='],
    [0x02, 3, 'AIBAE==='],
    [0x02, 4, 'AIBAEAQ='],
    [0x02, 5, 'AIBAEAQC'],
    [0x03, 1, 'AM======'],
    [0x03, 2, 'AMBQ===='],
    [0x03, 3, 'AMBQG==='],
    [0x03, 4, 'AMBQGAY='],
    [0x03, 5, 'AMBQGAYD'],
    [0x04, 1, 'AQ======'],
    [0x04, 2, 'AQCA===='],
    [0x04, 3, 'AQCAI==='],
    [0x04, 4, 'AQCAIBA='],
    [0x04, 5, 'AQCAIBAE'],
    [0x05, 1, 'AU======'],
    [0x05, 2, 'AUCQ===='],
    [0x05, 3, 'AUCQK==='],
    [0x05, 4, 'AUCQKBI='],
    [0x05, 5, 'AUCQKBIF'],
    [0x06, 1, 'AY======'],
    [0x06, 2, 'AYDA===='],
    [0x06, 3, 'AYDAM==='],
    [0x06, 4, 'AYDAMBQ='],
    [0x06, 5, 'AYDAMBQG'],
    [0x07, 1, 'A4======'],
    [0x07, 2, 'A4DQ===='],
    [0x07, 3, 'A4DQO==='],
    [0x07, 4, 'A4DQOBY='],
    [0x07, 5, 'A4DQOBYH'],
    [0x08, 1, 'BA======'],
    [0x08, 2, 'BAEA===='],
    [0x08, 3, 'BAEAQ==='],
    [0x08, 4, 'BAEAQCA='],
    [0x08, 5, 'BAEAQCAI'],
    [0x09, 1, 'BE======'],
    [0x09, 2, 'BEEQ===='],
    [0x09, 3, 'BEEQS==='],
    [0x09, 4, 'BEEQSCI='],
    [0x09, 5, 'BEEQSCIJ'],
    [0x0a, 1, 'BI======'],
    [0x0a, 2, 'BIFA===='],
    [0x0a, 3, 'BIFAU==='],
    [0x0a, 4, 'BIFAUCQ='],
    [0x0a, 5, 'BIFAUCQK'],
    [0x0b, 1, 'BM======'],
    [0x0b, 2, 'BMFQ===='],
    [0x0b, 3, 'BMFQW==='],
    [0x0b, 4, 'BMFQWCY='],
    [0x0b, 5, 'BMFQWCYL'],
    [0x0c, 1, 'BQ======'],
    [0x0c, 2, 'BQGA===='],
    [0x0c, 3, 'BQGAY==='],
    [0x0c, 4, 'BQGAYDA='],
    [0x0c, 5, 'BQGAYDAM'],
    [0x0d, 1, 'BU======'],
    [0x0d, 2, 'BUGQ===='],
    [0x0d, 3, 'BUGQ2==='],
    [0x0d, 4, 'BUGQ2DI='],
    [0x0d, 5, 'BUGQ2DIN'],
    [0x0e, 1, 'BY======'],
    [0x0e, 2, 'BYHA===='],
    [0x0e, 3, 'BYHA4==='],
    [0x0e, 4, 'BYHA4DQ='],
    [0x0e, 5, 'BYHA4DQO'],
    [0x0f, 1, 'B4======'],
    [0x0f, 2, 'B4HQ===='],
    [0x0f, 3, 'B4HQ6==='],
    [0x0f, 4, 'B4HQ6DY='],
    [0x0f, 5, 'B4HQ6DYP'],
    [0x10, 1, 'CA======'],
    [0x10, 2, 'CAIA===='],
    [0x10, 3, 'CAIBA==='],
    [0x10, 4, 'CAIBAEA='],
    [0x10, 5, 'CAIBAEAQ'],
    [0x11, 1, 'CE======'],
    [0x11, 2, 'CEIQ===='],
    [0x11, 3, 'CEIRC==='],
    [0x11, 4, 'CEIRCEI='],
    [0x11, 5, 'CEIRCEIR'],
    [0x12, 1, 'CI======'],
    [0x12, 2, 'CIJA===='],
    [0x12, 3, 'CIJBE==='],
    [0x12, 4, 'CIJBEEQ='],
    [0x12, 5, 'CIJBEEQS'],
    [0x13, 1, 'CM======'],
    [0x13, 2, 'CMJQ===='],
    [0x13, 3, 'CMJRG==='],
    [0x13, 4, 'CMJRGEY='],
    [0x13, 5, 'CMJRGEYT'],
    [0x14, 1, 'CQ======'],
    [0x14, 2, 'CQKA===='],
    [0x14, 3, 'CQKBI==='],
    [0x14, 4, 'CQKBIFA='],
    [0x14, 5, 'CQKBIFAU'],
    [0x15, 1, 'CU======'],
    [0x15, 2, 'CUKQ===='],
    [0x15, 3, 'CUKRK==='],
    [0x15, 4, 'CUKRKFI='],
    [0x15, 5, 'CUKRKFIV'],
    [0x16, 1, 'CY======'],
    [0x16, 2, 'CYLA===='],
    [0x16, 3, 'CYLBM==='],
    [0x16, 4, 'CYLBMFQ='],
    [0x16, 5, 'CYLBMFQW'],
    [0x17, 1, 'C4======'],
    [0x17, 2, 'C4LQ===='],
    [0x17, 3, 'C4LRO==='],
    [0x17, 4, 'C4LROFY='],
    [0x17, 5, 'C4LROFYX'],
    [0x18, 1, 'DA======'],
    [0x18, 2, 'DAMA===='],
    [0x18, 3, 'DAMBQ==='],
    [0x18, 4, 'DAMBQGA='],
    [0x18, 5, 'DAMBQGAY'],
    [0x19, 1, 'DE======'],
    [0x19, 2, 'DEMQ===='],
    [0x19, 3, 'DEMRS==='],
    [0x19, 4, 'DEMRSGI='],
    [0x19, 5, 'DEMRSGIZ'],
    [0x1a, 1, 'DI======'],
    [0x1a, 2, 'DINA===='],
    [0x1a, 3, 'DINBU==='],
    [0x1a, 4, 'DINBUGQ='],
    [0x1a, 5, 'DINBUGQ2'],
    [0x1b, 1, 'DM======'],
    [0x1b, 2, 'DMNQ===='],
    [0x1b, 3, 'DMNRW==='],
    [0x1b, 4, 'DMNRWGY='],
    [0x1b, 5, 'DMNRWGY3'],
    [0x1c, 1, 'DQ======'],
    [0x1c, 2, 'DQOA===='],
    [0x1c, 3, 'DQOBY==='],
    [0x1c, 4, 'DQOBYHA='],
    [0x1c, 5, 'DQOBYHA4'],
    [0x1d, 1, 'DU======'],
    [0x1d, 2, 'DUOQ===='],
    [0x1d, 3, 'DUOR2==='],
    [0x1d, 4, 'DUOR2HI='],
    [0x1d, 5, 'DUOR2HI5'],
    [0x1e, 1, 'DY======'],
    [0x1e, 2, 'DYPA===='],
    [0x1e, 3, 'DYPB4==='],
    [0x1e, 4, 'DYPB4HQ='],
    [0x1e, 5, 'DYPB4HQ6'],
    [0x1f, 1, 'D4======'],
    [0x1f, 2, 'D4PQ===='],
    [0x1f, 3, 'D4PR6==='],
    [0x1f, 4, 'D4PR6HY='],
    [0x1f, 5, 'D4PR6HY7'],
    [0x20, 1, 'EA======'],
    [0x20, 2, 'EAQA===='],
    [0x20, 3, 'EAQCA==='],
    [0x20, 4, 'EAQCAIA='],
    [0x20, 5, 'EAQCAIBA'],
    [0x21, 1, 'EE======'],
    [0x21, 2, 'EEQQ===='],
    [0x21, 3, 'EEQSC==='],
    [0x21, 4, 'EEQSCII='],
    [0x21, 5, 'EEQSCIJB'],
    [0x22, 1, 'EI======'],
    [0x22, 2, 'EIRA===='],
    [0x22, 3, 'EIRCE==='],
    [0x22, 4, 'EIRCEIQ='],
    [0x22, 5, 'EIRCEIRC'],
    [0x23, 1, 'EM======'],
    [0x23, 2, 'EMRQ===='],
    [0x23, 3, 'EMRSG==='],
    [0x23, 4, 'EMRSGIY='],
    [0x23, 5, 'EMRSGIZD'],
    [0x24, 1, 'EQ======'],
    [0x24, 2, 'EQSA===='],
    [0x24, 3, 'EQSCI==='],
    [0x24, 4, 'EQSCIJA='],
    [0x24, 5, 'EQSCIJBE'],
    [0x25, 1, 'EU======'],
    [0x25, 2, 'EUSQ===='],
    [0x25, 3, 'EUSSK==='],
    [0x25, 4, 'EUSSKJI='],
    [0x25, 5, 'EUSSKJJF'],
    [0x26, 1, 'EY======'],
    [0x26, 2, 'EYTA===='],
    [0x26, 3, 'EYTCM==='],
    [0x26, 4, 'EYTCMJQ='],
    [0x26, 5, 'EYTCMJRG'],
    [0x27, 1, 'E4======'],
    [0x27, 2, 'E4TQ===='],
    [0x27, 3, 'E4TSO==='],
    [0x27, 4, 'E4TSOJY='],
    [0x27, 5, 'E4TSOJZH'],
    [0x28, 1, 'FA======'],
    [0x28, 2, 'FAUA===='],
    [0x28, 3, 'FAUCQ==='],
    [0x28, 4, 'FAUCQKA='],
    [0x28, 5, 'FAUCQKBI'],
    [0x29, 1, 'FE======'],
    [0x29, 2, 'FEUQ===='],
    [0x29, 3, 'FEUSS==='],
    [0x29, 4, 'FEUSSKI='],
    [0x29, 5, 'FEUSSKJJ'],
    [0x2a, 1, 'FI======'],
    [0x2a, 2, 'FIVA===='],
    [0x2a, 3, 'FIVCU==='],
    [0x2a, 4, 'FIVCUKQ='],
    [0x2a, 5, 'FIVCUKRK'],
    [0x2b, 1, 'FM======'],
    [0x2b, 2, 'FMVQ===='],
    [0x2b, 3, 'FMVSW==='],
    [0x2b, 4, 'FMVSWKY='],
    [0x2b, 5, 'FMVSWKZL'],
    [0x2c, 1, 'FQ======'],
    [0x2c, 2, 'FQWA===='],
    [0x2c, 3, 'FQWCY==='],
    [0x2c, 4, 'FQWCYLA='],
    [0x2c, 5, 'FQWCYLBM'],
    [0x2d, 1, 'FU======'],
    [0x2d, 2, 'FUWQ===='],
    [0x2d, 3, 'FUWS2==='],
    [0x2d, 4, 'FUWS2LI='],
    [0x2d, 5, 'FUWS2LJN'],
    [0x2e, 1, 'FY======'],
    [0x2e, 2, 'FYXA===='],
    [0x2e, 3, 'FYXC4==='],
    [0x2e, 4, 'FYXC4LQ='],
    [0x2e, 5, 'FYXC4LRO'],
    [0x2f, 1, 'F4======'],
    [0x2f, 2, 'F4XQ===='],
    [0x2f, 3, 'F4XS6==='],
    [0x2f, 4, 'F4XS6LY='],
    [0x2f, 5, 'F4XS6LZP'],
    [0x30, 1, 'GA======'],
    [0x30, 2, 'GAYA===='],
    [0x30, 3, 'GAYDA==='],
    [0x30, 4, 'GAYDAMA='],
    [0x30, 5, 'GAYDAMBQ'],
    [0x31, 1, 'GE======'],
    [0x31, 2, 'GEYQ===='],
    [0x31, 3, 'GEYTC==='],
    [0x31, 4, 'GEYTCMI='],
    [0x31, 5, 'GEYTCMJR'],
    [0x32, 1, 'GI======'],
    [0x32, 2, 'GIZA===='],
    [0x32, 3, 'GIZDE==='],
    [0x32, 4, 'GIZDEMQ='],
    [0x32, 5, 'GIZDEMRS'],
    [0x33, 1, 'GM======'],
    [0x33, 2, 'GMZQ===='],
    [0x33, 3, 'GMZTG==='],
    [0x33, 4, 'GMZTGMY='],
    [0x33, 5, 'GMZTGMZT'],
    [0x34, 1, 'GQ======'],
    [0x34, 2, 'GQ2A===='],
    [0x34, 3, 'GQ2DI==='],
    [0x34, 4, 'GQ2DINA='],
    [0x34, 5, 'GQ2DINBU'],
    [0x35, 1, 'GU======'],
    [0x35, 2, 'GU2Q===='],
    [0x35, 3, 'GU2TK==='],
    [0x35, 4, 'GU2TKNI='],
    [0x35, 5, 'GU2TKNJV'],
    [0x36, 1, 'GY======'],
    [0x36, 2, 'GY3A===='],
    [0x36, 3, 'GY3DM==='],
    [0x36, 4, 'GY3DMNQ='],
    [0x36, 5, 'GY3DMNRW'],
    [0x37, 1, 'G4======'],
    [0x37, 2, 'G43Q===='],
    [0x37, 3, 'G43TO==='],
    [0x37, 4, 'G43TONY='],
    [0x37, 5, 'G43TONZX'],
    [0x38, 1, 'HA======'],
    [0x38, 2, 'HA4A===='],
    [0x38, 3, 'HA4DQ==='],
    [0x38, 4, 'HA4DQOA='],
    [0x38, 5, 'HA4DQOBY'],
    [0x39, 1, 'HE======'],
    [0x39, 2, 'HE4Q===='],
    [0x39, 3, 'HE4TS==='],
    [0x39, 4, 'HE4TSOI='],
    [0x39, 5, 'HE4TSOJZ'],
    [0x3a, 1, 'HI======'],
    [0x3a, 2, 'HI5A===='],
    [0x3a, 3, 'HI5DU==='],
    [0x3a, 4, 'HI5DUOQ='],
    [0x3a, 5, 'HI5DUOR2'],
    [0x3b, 1, 'HM======'],
    [0x3b, 2, 'HM5Q===='],
    [0x3b, 3, 'HM5TW==='],
    [0x3b, 4, 'HM5TWOY='],
    [0x3b, 5, 'HM5TWOZ3'],
    [0x3c, 1, 'HQ======'],
    [0x3c, 2, 'HQ6A===='],
    [0x3c, 3, 'HQ6DY==='],
    [0x3c, 4, 'HQ6DYPA='],
    [0x3c, 5, 'HQ6DYPB4'],
    [0x3d, 1, 'HU======'],
    [0x3d, 2, 'HU6Q===='],
    [0x3d, 3, 'HU6T2==='],
    [0x3d, 4, 'HU6T2PI='],
    [0x3d, 5, 'HU6T2PJ5'],
    [0x3e, 1, 'HY======'],
    [0x3e, 2, 'HY7A===='],
    [0x3e, 3, 'HY7D4==='],
    [0x3e, 4, 'HY7D4PQ='],
    [0x3e, 5, 'HY7D4PR6'],
    [0x3f, 1, 'H4======'],
    [0x3f, 2, 'H47Q===='],
    [0x3f, 3, 'H47T6==='],
    [0x3f, 4, 'H47T6PY='],
    [0x3f, 5, 'H47T6PZ7'],
    [0x40, 1, 'IA======'],
    [0x40, 2, 'IBAA===='],
    [0x40, 3, 'IBAEA==='],
    [0x40, 4, 'IBAEAQA='],
    [0x40, 5, 'IBAEAQCA'],
    [0x41, 1, 'IE======'],
    [0x41, 2, 'IFAQ===='],
    [0x41, 3, 'IFAUC==='],
    [0x41, 4, 'IFAUCQI='],
    [0x41, 5, 'IFAUCQKB'],
    [0x42, 1, 'II======'],
    [0x42, 2, 'IJBA===='],
    [0x42, 3, 'IJBEE==='],
    [0x42, 4, 'IJBEEQQ='],
    [0x42, 5, 'IJBEEQSC'],
    [0x43, 1, 'IM======'],
    [0x43, 2, 'INBQ===='],
    [0x43, 3, 'INBUG==='],
    [0x43, 4, 'INBUGQY='],
    [0x43, 5, 'INBUGQ2D'],
    [0x44, 1, 'IQ======'],
    [0x44, 2, 'IRCA===='],
    [0x44, 3, 'IRCEI==='],
    [0x44, 4, 'IRCEIRA='],
    [0x44, 5, 'IRCEIRCE'],
    [0x45, 1, 'IU======'],
    [0x45, 2, 'IVCQ===='],
    [0x45, 3, 'IVCUK==='],
    [0x45, 4, 'IVCUKRI='],
    [0x45, 5, 'IVCUKRKF'],
    [0x46, 1, 'IY======'],
    [0x46, 2, 'IZDA===='],
    [0x46, 3, 'IZDEM==='],
    [0x46, 4, 'IZDEMRQ='],
    [0x46, 5, 'IZDEMRSG'],
    [0x47, 1, 'I4======'],
    [0x47, 2, 'I5DQ===='],
    [0x47, 3, 'I5DUO==='],
    [0x47, 4, 'I5DUORY='],
    [0x47, 5, 'I5DUOR2H'],
    [0x48, 1, 'JA======'],
    [0x48, 2, 'JBEA===='],
    [0x48, 3, 'JBEEQ==='],
    [0x48, 4, 'JBEEQSA='],
    [0x48, 5, 'JBEEQSCI'],
    [0x49, 1, 'JE======'],
    [0x49, 2, 'JFEQ===='],
    [0x49, 3, 'JFEUS==='],
    [0x49, 4, 'JFEUSSI='],
    [0x49, 5, 'JFEUSSKJ'],
    [0x4a, 1, 'JI======'],
    [0x4a, 2, 'JJFA===='],
    [0x4a, 3, 'JJFEU==='],
    [0x4a, 4, 'JJFEUSQ='],
    [0x4a, 5, 'JJFEUSSK'],
    [0x4b, 1, 'JM======'],
    [0x4b, 2, 'JNFQ===='],
    [0x4b, 3, 'JNFUW==='],
    [0x4b, 4, 'JNFUWSY='],
    [0x4b, 5, 'JNFUWS2L'],
    [0x4c, 1, 'JQ======'],
    [0x4c, 2, 'JRGA===='],
    [0x4c, 3, 'JRGEY==='],
    [0x4c, 4, 'JRGEYTA='],
    [0x4c, 5, 'JRGEYTCM'],
    [0x4d, 1, 'JU======'],
    [0x4d, 2, 'JVGQ===='],
    [0x4d, 3, 'JVGU2==='],
    [0x4d, 4, 'JVGU2TI='],
    [0x4d, 5, 'JVGU2TKN'],
    [0x4e, 1, 'JY======'],
    [0x4e, 2, 'JZHA===='],
    [0x4e, 3, 'JZHE4==='],
    [0x4e, 4, 'JZHE4TQ='],
    [0x4e, 5, 'JZHE4TSO'],
    [0x4f, 1, 'J4======'],
    [0x4f, 2, 'J5HQ===='],
    [0x4f, 3, 'J5HU6==='],
    [0x4f, 4, 'J5HU6TY='],
    [0x4f, 5, 'J5HU6T2P'],
    [0x50, 1, 'KA======'],
    [0x50, 2, 'KBIA===='],
    [0x50, 3, 'KBIFA==='],
    [0x50, 4, 'KBIFAUA='],
    [0x50, 5, 'KBIFAUCQ'],
    [0x51, 1, 'KE======'],
    [0x51, 2, 'KFIQ===='],
    [0x51, 3, 'KFIVC==='],
    [0x51, 4, 'KFIVCUI='],
    [0x51, 5, 'KFIVCUKR'],
    [0x52, 1, 'KI======'],
    [0x52, 2, 'KJJA===='],
    [0x52, 3, 'KJJFE==='],
    [0x52, 4, 'KJJFEUQ='],
    [0x52, 5, 'KJJFEUSS'],
    [0x53, 1, 'KM======'],
    [0x53, 2, 'KNJQ===='],
    [0x53, 3, 'KNJVG==='],
    [0x53, 4, 'KNJVGUY='],
    [0x53, 5, 'KNJVGU2T'],
    [0x54, 1, 'KQ======'],
    [0x54, 2, 'KRKA===='],
    [0x54, 3, 'KRKFI==='],
    [0x54, 4, 'KRKFIVA='],
    [0x54, 5, 'KRKFIVCU'],
    [0x55, 1, 'KU======'],
    [0x55, 2, 'KVKQ===='],
    [0x55, 3, 'KVKVK==='],
    [0x55, 4, 'KVKVKVI='],
    [0x55, 5, 'KVKVKVKV'],
    [0x56, 1, 'KY======'],
    [0x56, 2, 'KZLA===='],
    [0x56, 3, 'KZLFM==='],
    [0x56, 4, 'KZLFMVQ='],
    [0x56, 5, 'KZLFMVSW'],
    [0x57, 1, 'K4======'],
    [0x57, 2, 'K5LQ===='],
    [0x57, 3, 'K5LVO==='],
    [0x57, 4, 'K5LVOVY='],
    [0x57, 5, 'K5LVOV2X'],
    [0x58, 1, 'LA======'],
    [0x58, 2, 'LBMA===='],
    [0x58, 3, 'LBMFQ==='],
    [0x58, 4, 'LBMFQWA='],
    [0x58, 5, 'LBMFQWCY'],
    [0x59, 1, 'LE======'],
    [0x59, 2, 'LFMQ===='],
    [0x59, 3, 'LFMVS==='],
    [0x59, 4, 'LFMVSWI='],
    [0x59, 5, 'LFMVSWKZ'],
    [0x5a, 1, 'LI======'],
    [0x5a, 2, 'LJNA===='],
    [0x5a, 3, 'LJNFU==='],
    [0x5a, 4, 'LJNFUWQ='],
    [0x5a, 5, 'LJNFUWS2'],
    [0x5b, 1, 'LM======'],
    [0x5b, 2, 'LNNQ===='],
    [0x5b, 3, 'LNNVW==='],
    [0x5b, 4, 'LNNVWWY='],
    [0x5b, 5, 'LNNVWW23'],
    [0x5c, 1, 'LQ======'],
    [0x5c, 2, 'LROA===='],
    [0x5c, 3, 'LROFY==='],
    [0x5c, 4, 'LROFYXA='],
    [0x5c, 5, 'LROFYXC4'],
    [0x5d, 1, 'LU======'],
    [0x5d, 2, 'LVOQ===='],
    [0x5d, 3, 'LVOV2==='],
    [0x5d, 4, 'LVOV2XI='],
    [0x5d, 5, 'LVOV2XK5'],
    [0x5e, 1, 'LY======'],
    [0x5e, 2, 'LZPA===='],
    [0x5e, 3, 'LZPF4==='],
    [0x5e, 4, 'LZPF4XQ='],
    [0x5e, 5, 'LZPF4XS6'],
    [0x5f, 1, 'L4======'],
    [0x5f, 2, 'L5PQ===='],
    [0x5f, 3, 'L5PV6==='],
    [0x5f, 4, 'L5PV6XY='],
    [0x5f, 5, 'L5PV6X27'],
    [0x60, 1, 'MA======'],
    [0x60, 2, 'MBQA===='],
    [0x60, 3, 'MBQGA==='],
    [0x60, 4, 'MBQGAYA='],
    [0x60, 5, 'MBQGAYDA'],
    [0x61, 1, 'ME======'],
    [0x61, 2, 'MFQQ===='],
    [0x61, 3, 'MFQWC==='],
    [0x61, 4, 'MFQWCYI='],
    [0x61, 5, 'MFQWCYLB'],
    [0x62, 1, 'MI======'],
    [0x62, 2, 'MJRA===='],
    [0x62, 3, 'MJRGE==='],
    [0x62, 4, 'MJRGEYQ='],
    [0x62, 5, 'MJRGEYTC'],
    [0x63, 1, 'MM======'],
    [0x63, 2, 'MNRQ===='],
    [0x63, 3, 'MNRWG==='],
    [0x63, 4, 'MNRWGYY='],
    [0x63, 5, 'MNRWGY3D'],
    [0x64, 1, 'MQ======'],
    [0x64, 2, 'MRSA===='],
    [0x64, 3, 'MRSGI==='],
    [0x64, 4, 'MRSGIZA='],
    [0x64, 5, 'MRSGIZDE'],
    [0x65, 1, 'MU======'],
    [0x65, 2, 'MVSQ===='],
    [0x65, 3, 'MVSWK==='],
    [0x65, 4, 'MVSWKZI='],
    [0x65, 5, 'MVSWKZLF'],
    [0x66, 1, 'MY======'],
    [0x66, 2, 'MZTA===='],
    [0x66, 3, 'MZTGM==='],
    [0x66, 4, 'MZTGMZQ='],
    [0x66, 5, 'MZTGMZTG'],
    [0x67, 1, 'M4======'],
    [0x67, 2, 'M5TQ===='],
    [0x67, 3, 'M5TWO==='],
    [0x67, 4, 'M5TWOZY='],
    [0x67, 5, 'M5TWOZ3H'],
    [0x68, 1, 'NA======'],
    [0x68, 2, 'NBUA===='],
    [0x68, 3, 'NBUGQ==='],
    [0x68, 4, 'NBUGQ2A='],
    [0x68, 5, 'NBUGQ2DI'],
    [0x69, 1, 'NE======'],
    [0x69, 2, 'NFUQ===='],
    [0x69, 3, 'NFUWS==='],
    [0x69, 4, 'NFUWS2I='],
    [0x69, 5, 'NFUWS2LJ'],
    [0x6a, 1, 'NI======'],
    [0x6a, 2, 'NJVA===='],
    [0x6a, 3, 'NJVGU==='],
    [0x6a, 4, 'NJVGU2Q='],
    [0x6a, 5, 'NJVGU2TK'],
    [0x6b, 1, 'NM======'],
    [0x6b, 2, 'NNVQ===='],
    [0x6b, 3, 'NNVWW==='],
    [0x6b, 4, 'NNVWW2Y='],
    [0x6b, 5, 'NNVWW23L'],
    [0x6c, 1, 'NQ======'],
    [0x6c, 2, 'NRWA===='],
    [0x6c, 3, 'NRWGY==='],
    [0x6c, 4, 'NRWGY3A='],
    [0x6c, 5, 'NRWGY3DM'],
    [0x6d, 1, 'NU======'],
    [0x6d, 2, 'NVWQ===='],
    [0x6d, 3, 'NVWW2==='],
    [0x6d, 4, 'NVWW23I='],
    [0x6d, 5, 'NVWW23LN'],
    [0x6e, 1, 'NY======'],
    [0x6e, 2, 'NZXA===='],
    [0x6e, 3, 'NZXG4==='],
    [0x6e, 4, 'NZXG43Q='],
    [0x6e, 5, 'NZXG43TO'],
    [0x6f, 1, 'N4======'],
    [0x6f, 2, 'N5XQ===='],
    [0x6f, 3, 'N5XW6==='],
    [0x6f, 4, 'N5XW63Y='],
    [0x6f, 5, 'N5XW633P'],
    [0x70, 1, 'OA======'],
    [0x70, 2, 'OBYA===='],
    [0x70, 3, 'OBYHA==='],
    [0x70, 4, 'OBYHA4A='],
    [0x70, 5, 'OBYHA4DQ'],
    [0x71, 1, 'OE======'],
    [0x71, 2, 'OFYQ===='],
    [0x71, 3, 'OFYXC==='],
    [0x71, 4, 'OFYXC4I='],
    [0x71, 5, 'OFYXC4LR'],
    [0x72, 1, 'OI======'],
    [0x72, 2, 'OJZA===='],
    [0x72, 3, 'OJZHE==='],
    [0x72, 4, 'OJZHE4Q='],
    [0x72, 5, 'OJZHE4TS'],
    [0x73, 1, 'OM======'],
    [0x73, 2, 'ONZQ===='],
    [0x73, 3, 'ONZXG==='],
    [0x73, 4, 'ONZXG4Y='],
    [0x73, 5, 'ONZXG43T'],
    [0x74, 1, 'OQ======'],
    [0x74, 2, 'OR2A===='],
    [0x74, 3, 'OR2HI==='],
    [0x74, 4, 'OR2HI5A='],
    [0x74, 5, 'OR2HI5DU'],
    [0x75, 1, 'OU======'],
    [0x75, 2, 'OV2Q===='],
    [0x75, 3, 'OV2XK==='],
    [0x75, 4, 'OV2XK5I='],
    [0x75, 5, 'OV2XK5LV'],
    [0x76, 1, 'OY======'],
    [0x76, 2, 'OZ3A===='],
    [0x76, 3, 'OZ3HM==='],
    [0x76, 4, 'OZ3HM5Q='],
    [0x76, 5, 'OZ3HM5TW'],
    [0x77, 1, 'O4======'],
    [0x77, 2, 'O53Q===='],
    [0x77, 3, 'O53XO==='],
    [0x77, 4, 'O53XO5Y='],
    [0x77, 5, 'O53XO53X'],
    [0x78, 1, 'PA======'],
    [0x78, 2, 'PB4A===='],
    [0x78, 3, 'PB4HQ==='],
    [0x78, 4, 'PB4HQ6A='],
    [0x78, 5, 'PB4HQ6DY'],
    [0x79, 1, 'PE======'],
    [0x79, 2, 'PF4Q===='],
    [0x79, 3, 'PF4XS==='],
    [0x79, 4, 'PF4XS6I='],
    [0x79, 5, 'PF4XS6LZ'],
    [0x7a, 1, 'PI======'],
    [0x7a, 2, 'PJ5A===='],
    [0x7a, 3, 'PJ5HU==='],
    [0x7a, 4, 'PJ5HU6Q='],
    [0x7a, 5, 'PJ5HU6T2'],
    [0x7b, 1, 'PM======'],
    [0x7b, 2, 'PN5Q===='],
    [0x7b, 3, 'PN5XW==='],
    [0x7b, 4, 'PN5XW6Y='],
    [0x7b, 5, 'PN5XW633'],
    [0x7c, 1, 'PQ======'],
    [0x7c, 2, 'PR6A===='],
    [0x7c, 3, 'PR6HY==='],
    [0x7c, 4, 'PR6HY7A='],
    [0x7c, 5, 'PR6HY7D4'],
    [0x7d, 1, 'PU======'],
    [0x7d, 2, 'PV6Q===='],
    [0x7d, 3, 'PV6X2==='],
    [0x7d, 4, 'PV6X27I='],
    [0x7d, 5, 'PV6X27L5'],
    [0x7e, 1, 'PY======'],
    [0x7e, 2, 'PZ7A===='],
    [0x7e, 3, 'PZ7H4==='],
    [0x7e, 4, 'PZ7H47Q='],
    [0x7e, 5, 'PZ7H47T6'],
    [0x7f, 1, 'P4======'],
    [0x7f, 2, 'P57Q===='],
    [0x7f, 3, 'P57X6==='],
    [0x7f, 4, 'P57X67Y='],
    [0x7f, 5, 'P57X6737'],
    [0x80, 1, 'QA======'],
    [0x80, 2, 'QCAA===='],
    [0x80, 3, 'QCAIA==='],
    [0x80, 4, 'QCAIBAA='],
    [0x80, 5, 'QCAIBAEA'],
    [0x81, 1, 'QE======'],
    [0x81, 2, 'QGAQ===='],
    [0x81, 3, 'QGAYC==='],
    [0x81, 4, 'QGAYDAI='],
    [0x81, 5, 'QGAYDAMB'],
    [0x82, 1, 'QI======'],
    [0x82, 2, 'QKBA===='],
    [0x82, 3, 'QKBIE==='],
    [0x82, 4, 'QKBIFAQ='],
    [0x82, 5, 'QKBIFAUC'],
    [0x83, 1, 'QM======'],
    [0x83, 2, 'QOBQ===='],
    [0x83, 3, 'QOBYG==='],
    [0x83, 4, 'QOBYHAY='],
    [0x83, 5, 'QOBYHA4D'],
    [0x84, 1, 'QQ======'],
    [0x84, 2, 'QSCA===='],
    [0x84, 3, 'QSCII==='],
    [0x84, 4, 'QSCIJBA='],
    [0x84, 5, 'QSCIJBEE'],
    [0x85, 1, 'QU======'],
    [0x85, 2, 'QWCQ===='],
    [0x85, 3, 'QWCYK==='],
    [0x85, 4, 'QWCYLBI='],
    [0x85, 5, 'QWCYLBMF'],
    [0x86, 1, 'QY======'],
    [0x86, 2, 'Q2DA===='],
    [0x86, 3, 'Q2DIM==='],
    [0x86, 4, 'Q2DINBQ='],
    [0x86, 5, 'Q2DINBUG'],
    [0x87, 1, 'Q4======'],
    [0x87, 2, 'Q6DQ===='],
    [0x87, 3, 'Q6DYO==='],
    [0x87, 4, 'Q6DYPBY='],
    [0x87, 5, 'Q6DYPB4H'],
    [0x88, 1, 'RA======'],
    [0x88, 2, 'RCEA===='],
    [0x88, 3, 'RCEIQ==='],
    [0x88, 4, 'RCEIRCA='],
    [0x88, 5, 'RCEIRCEI'],
    [0x89, 1, 'RE======'],
    [0x89, 2, 'RGEQ===='],
    [0x89, 3, 'RGEYS==='],
    [0x89, 4, 'RGEYTCI='],
    [0x89, 5, 'RGEYTCMJ'],
    [0x8a, 1, 'RI======'],
    [0x8a, 2, 'RKFA===='],
    [0x8a, 3, 'RKFIU==='],
    [0x8a, 4, 'RKFIVCQ='],
    [0x8a, 5, 'RKFIVCUK'],
    [0x8b, 1, 'RM======'],
    [0x8b, 2, 'ROFQ===='],
    [0x8b, 3, 'ROFYW==='],
    [0x8b, 4, 'ROFYXCY='],
    [0x8b, 5, 'ROFYXC4L'],
    [0x8c, 1, 'RQ======'],
    [0x8c, 2, 'RSGA===='],
    [0x8c, 3, 'RSGIY==='],
    [0x8c, 4, 'RSGIZDA='],
    [0x8c, 5, 'RSGIZDEM'],
    [0x8d, 1, 'RU======'],
    [0x8d, 2, 'RWGQ===='],
    [0x8d, 3, 'RWGY2==='],
    [0x8d, 4, 'RWGY3DI='],
    [0x8d, 5, 'RWGY3DMN'],
    [0x8e, 1, 'RY======'],
    [0x8e, 2, 'R2HA===='],
    [0x8e, 3, 'R2HI4==='],
    [0x8e, 4, 'R2HI5DQ='],
    [0x8e, 5, 'R2HI5DUO'],
    [0x8f, 1, 'R4======'],
    [0x8f, 2, 'R6HQ===='],
    [0x8f, 3, 'R6HY6==='],
    [0x8f, 4, 'R6HY7DY='],
    [0x8f, 5, 'R6HY7D4P'],
    [0x90, 1, 'SA======'],
    [0x90, 2, 'SCIA===='],
    [0x90, 3, 'SCIJA==='],
    [0x90, 4, 'SCIJBEA='],
    [0x90, 5, 'SCIJBEEQ'],
    [0x91, 1, 'SE======'],
    [0x91, 2, 'SGIQ===='],
    [0x91, 3, 'SGIZC==='],
    [0x91, 4, 'SGIZDEI='],
    [0x91, 5, 'SGIZDEMR'],
    [0x92, 1, 'SI======'],
    [0x92, 2, 'SKJA===='],
    [0x92, 3, 'SKJJE==='],
    [0x92, 4, 'SKJJFEQ='],
    [0x92, 5, 'SKJJFEUS'],
    [0x93, 1, 'SM======'],
    [0x93, 2, 'SOJQ===='],
    [0x93, 3, 'SOJZG==='],
    [0x93, 4, 'SOJZHEY='],
    [0x93, 5, 'SOJZHE4T'],
    [0x94, 1, 'SQ======'],
    [0x94, 2, 'SSKA===='],
    [0x94, 3, 'SSKJI==='],
    [0x94, 4, 'SSKJJFA='],
    [0x94, 5, 'SSKJJFEU'],
    [0x95, 1, 'SU======'],
    [0x95, 2, 'SWKQ===='],
    [0x95, 3, 'SWKZK==='],
    [0x95, 4, 'SWKZLFI='],
    [0x95, 5, 'SWKZLFMV'],
    [0x96, 1, 'SY======'],
    [0x96, 2, 'S2LA===='],
    [0x96, 3, 'S2LJM==='],
    [0x96, 4, 'S2LJNFQ='],
    [0x96, 5, 'S2LJNFUW'],
    [0x97, 1, 'S4======'],
    [0x97, 2, 'S6LQ===='],
    [0x97, 3, 'S6LZO==='],
    [0x97, 4, 'S6LZPFY='],
    [0x97, 5, 'S6LZPF4X'],
    [0x98, 1, 'TA======'],
    [0x98, 2, 'TCMA===='],
    [0x98, 3, 'TCMJQ==='],
    [0x98, 4, 'TCMJRGA='],
    [0x98, 5, 'TCMJRGEY'],
    [0x99, 1, 'TE======'],
    [0x99, 2, 'TGMQ===='],
    [0x99, 3, 'TGMZS==='],
    [0x99, 4, 'TGMZTGI='],
    [0x99, 5, 'TGMZTGMZ'],
    [0x9a, 1, 'TI======'],
    [0x9a, 2, 'TKNA===='],
    [0x9a, 3, 'TKNJU==='],
    [0x9a, 4, 'TKNJVGQ='],
    [0x9a, 5, 'TKNJVGU2'],
    [0x9b, 1, 'TM======'],
    [0x9b, 2, 'TONQ===='],
    [0x9b, 3, 'TONZW==='],
    [0x9b, 4, 'TONZXGY='],
    [0x9b, 5, 'TONZXG43'],
    [0x9c, 1, 'TQ======'],
    [0x9c, 2, 'TSOA===='],
    [0x9c, 3, 'TSOJY==='],
    [0x9c, 4, 'TSOJZHA='],
    [0x9c, 5, 'TSOJZHE4'],
    [0x9d, 1, 'TU======'],
    [0x9d, 2, 'TWOQ===='],
    [0x9d, 3, 'TWOZ2==='],
    [0x9d, 4, 'TWOZ3HI='],
    [0x9d, 5, 'TWOZ3HM5'],
    [0x9e, 1, 'TY======'],
    [0x9e, 2, 'T2PA===='],
    [0x9e, 3, 'T2PJ4==='],
    [0x9e, 4, 'T2PJ5HQ='],
    [0x9e, 5, 'T2PJ5HU6'],
    [0x9f, 1, 'T4======'],
    [0x9f, 2, 'T6PQ===='],
    [0x9f, 3, 'T6PZ6==='],
    [0x9f, 4, 'T6PZ7HY='],
    [0x9f, 5, 'T6PZ7H47'],
    [0xa0, 1, 'UA======'],
    [0xa0, 2, 'UCQA===='],
    [0xa0, 3, 'UCQKA==='],
    [0xa0, 4, 'UCQKBIA='],
    [0xa0, 5, 'UCQKBIFA'],
    [0xa1, 1, 'UE======'],
    [0xa1, 2, 'UGQQ===='],
    [0xa1, 3, 'UGQ2C==='],
    [0xa1, 4, 'UGQ2DII='],
    [0xa1, 5, 'UGQ2DINB'],
    [0xa2, 1, 'UI======'],
    [0xa2, 2, 'UKRA===='],
    [0xa2, 3, 'UKRKE==='],
    [0xa2, 4, 'UKRKFIQ='],
    [0xa2, 5, 'UKRKFIVC'],
    [0xa3, 1, 'UM======'],
    [0xa3, 2, 'UORQ===='],
    [0xa3, 3, 'UOR2G==='],
    [0xa3, 4, 'UOR2HIY='],
    [0xa3, 5, 'UOR2HI5D'],
    [0xa4, 1, 'UQ======'],
    [0xa4, 2, 'USSA===='],
    [0xa4, 3, 'USSKI==='],
    [0xa4, 4, 'USSKJJA='],
    [0xa4, 5, 'USSKJJFE'],
    [0xa5, 1, 'UU======'],
    [0xa5, 2, 'UWSQ===='],
    [0xa5, 3, 'UWS2K==='],
    [0xa5, 4, 'UWS2LJI='],
    [0xa5, 5, 'UWS2LJNF'],
    [0xa6, 1, 'UY======'],
    [0xa6, 2, 'U2TA===='],
    [0xa6, 3, 'U2TKM==='],
    [0xa6, 4, 'U2TKNJQ='],
    [0xa6, 5, 'U2TKNJVG'],
    [0xa7, 1, 'U4======'],
    [0xa7, 2, 'U6TQ===='],
    [0xa7, 3, 'U6T2O==='],
    [0xa7, 4, 'U6T2PJY='],
    [0xa7, 5, 'U6T2PJ5H'],
    [0xa8, 1, 'VA======'],
    [0xa8, 2, 'VCUA===='],
    [0xa8, 3, 'VCUKQ==='],
    [0xa8, 4, 'VCUKRKA='],
    [0xa8, 5, 'VCUKRKFI'],
    [0xa9, 1, 'VE======'],
    [0xa9, 2, 'VGUQ===='],
    [0xa9, 3, 'VGU2S==='],
    [0xa9, 4, 'VGU2TKI='],
    [0xa9, 5, 'VGU2TKNJ'],
    [0xaa, 1, 'VI======'],
    [0xaa, 2, 'VKVA===='],
    [0xaa, 3, 'VKVKU==='],
    [0xaa, 4, 'VKVKVKQ='],
    [0xaa, 5, 'VKVKVKVK'],
    [0xab, 1, 'VM======'],
    [0xab, 2, 'VOVQ===='],
    [0xab, 3, 'VOV2W==='],
    [0xab, 4, 'VOV2XKY='],
    [0xab, 5, 'VOV2XK5L'],
    [0xac, 1, 'VQ======'],
    [0xac, 2, 'VSWA===='],
    [0xac, 3, 'VSWKY==='],
    [0xac, 4, 'VSWKZLA='],
    [0xac, 5, 'VSWKZLFM'],
    [0xad, 1, 'VU======'],
    [0xad, 2, 'VWWQ===='],
    [0xad, 3, 'VWW22==='],
    [0xad, 4, 'VWW23LI='],
    [0xad, 5, 'VWW23LNN'],
    [0xae, 1, 'VY======'],
    [0xae, 2, 'V2XA===='],
    [0xae, 3, 'V2XK4==='],
    [0xae, 4, 'V2XK5LQ='],
    [0xae, 5, 'V2XK5LVO'],
    [0xaf, 1, 'V4======'],
    [0xaf, 2, 'V6XQ===='],
    [0xaf, 3, 'V6X26==='],
    [0xaf, 4, 'V6X27LY='],
    [0xaf, 5, 'V6X27L5P'],
    [0xb0, 1, 'WA======'],
    [0xb0, 2, 'WCYA===='],
    [0xb0, 3, 'WCYLA==='],
    [0xb0, 4, 'WCYLBMA='],
    [0xb0, 5, 'WCYLBMFQ'],
    [0xb1, 1, 'WE======'],
    [0xb1, 2, 'WGYQ===='],
    [0xb1, 3, 'WGY3C==='],
    [0xb1, 4, 'WGY3DMI='],
    [0xb1, 5, 'WGY3DMNR'],
    [0xb2, 1, 'WI======'],
    [0xb2, 2, 'WKZA===='],
    [0xb2, 3, 'WKZLE==='],
    [0xb2, 4, 'WKZLFMQ='],
    [0xb2, 5, 'WKZLFMVS'],
    [0xb3, 1, 'WM======'],
    [0xb3, 2, 'WOZQ===='],
    [0xb3, 3, 'WOZ3G==='],
    [0xb3, 4, 'WOZ3HMY='],
    [0xb3, 5, 'WOZ3HM5T'],
    [0xb4, 1, 'WQ======'],
    [0xb4, 2, 'WS2A===='],
    [0xb4, 3, 'WS2LI==='],
    [0xb4, 4, 'WS2LJNA='],
    [0xb4, 5, 'WS2LJNFU'],
    [0xb5, 1, 'WU======'],
    [0xb5, 2, 'WW2Q===='],
    [0xb5, 3, 'WW23K==='],
    [0xb5, 4, 'WW23LNI='],
    [0xb5, 5, 'WW23LNNV'],
    [0xb6, 1, 'WY======'],
    [0xb6, 2, 'W23A===='],
    [0xb6, 3, 'W23LM==='],
    [0xb6, 4, 'W23LNNQ='],
    [0xb6, 5, 'W23LNNVW'],
    [0xb7, 1, 'W4======'],
    [0xb7, 2, 'W63Q===='],
    [0xb7, 3, 'W633O==='],
    [0xb7, 4, 'W633PNY='],
    [0xb7, 5, 'W633PN5X'],
    [0xb8, 1, 'XA======'],
    [0xb8, 2, 'XC4A===='],
    [0xb8, 3, 'XC4LQ==='],
    [0xb8, 4, 'XC4LROA='],
    [0xb8, 5, 'XC4LROFY'],
    [0xb9, 1, 'XE======'],
    [0xb9, 2, 'XG4Q===='],
    [0xb9, 3, 'XG43S==='],
    [0xb9, 4, 'XG43TOI='],
    [0xb9, 5, 'XG43TONZ'],
    [0xba, 1, 'XI======'],
    [0xba, 2, 'XK5A===='],
    [0xba, 3, 'XK5LU==='],
    [0xba, 4, 'XK5LVOQ='],
    [0xba, 5, 'XK5LVOV2'],
    [0xbb, 1, 'XM======'],
    [0xbb, 2, 'XO5Q===='],
    [0xbb, 3, 'XO53W==='],
    [0xbb, 4, 'XO53XOY='],
    [0xbb, 5, 'XO53XO53'],
    [0xbc, 1, 'XQ======'],
    [0xbc, 2, 'XS6A===='],
    [0xbc, 3, 'XS6LY==='],
    [0xbc, 4, 'XS6LZPA='],
    [0xbc, 5, 'XS6LZPF4'],
    [0xbd, 1, 'XU======'],
    [0xbd, 2, 'XW6Q===='],
    [0xbd, 3, 'XW632==='],
    [0xbd, 4, 'XW633PI='],
    [0xbd, 5, 'XW633PN5'],
    [0xbe, 1, 'XY======'],
    [0xbe, 2, 'X27A===='],
    [0xbe, 3, 'X27L4==='],
    [0xbe, 4, 'X27L5PQ='],
    [0xbe, 5, 'X27L5PV6'],
    [0xbf, 1, 'X4======'],
    [0xbf, 2, 'X67Q===='],
    [0xbf, 3, 'X6736==='],
    [0xbf, 4, 'X6737PY='],
    [0xbf, 5, 'X6737P57'],
    [0xc0, 1, 'YA======'],
    [0xc0, 2, 'YDAA===='],
    [0xc0, 3, 'YDAMA==='],
    [0xc0, 4, 'YDAMBQA='],
    [0xc0, 5, 'YDAMBQGA'],
    [0xc1, 1, 'YE======'],
    [0xc1, 2, 'YHAQ===='],
    [0xc1, 3, 'YHA4C==='],
    [0xc1, 4, 'YHA4DQI='],
    [0xc1, 5, 'YHA4DQOB'],
    [0xc2, 1, 'YI======'],
    [0xc2, 2, 'YLBA===='],
    [0xc2, 3, 'YLBME==='],
    [0xc2, 4, 'YLBMFQQ='],
    [0xc2, 5, 'YLBMFQWC'],
    [0xc3, 1, 'YM======'],
    [0xc3, 2, 'YPBQ===='],
    [0xc3, 3, 'YPB4G==='],
    [0xc3, 4, 'YPB4HQY='],
    [0xc3, 5, 'YPB4HQ6D'],
    [0xc4, 1, 'YQ======'],
    [0xc4, 2, 'YTCA===='],
    [0xc4, 3, 'YTCMI==='],
    [0xc4, 4, 'YTCMJRA='],
    [0xc4, 5, 'YTCMJRGE'],
    [0xc5, 1, 'YU======'],
    [0xc5, 2, 'YXCQ===='],
    [0xc5, 3, 'YXC4K==='],
    [0xc5, 4, 'YXC4LRI='],
    [0xc5, 5, 'YXC4LROF'],
    [0xc6, 1, 'YY======'],
    [0xc6, 2, 'Y3DA===='],
    [0xc6, 3, 'Y3DMM==='],
    [0xc6, 4, 'Y3DMNRQ='],
    [0xc6, 5, 'Y3DMNRWG'],
    [0xc7, 1, 'Y4======'],
    [0xc7, 2, 'Y7DQ===='],
    [0xc7, 3, 'Y7D4O==='],
    [0xc7, 4, 'Y7D4PRY='],
    [0xc7, 5, 'Y7D4PR6H'],
    [0xc8, 1, 'ZA======'],
    [0xc8, 2, 'ZDEA===='],
    [0xc8, 3, 'ZDEMQ==='],
    [0xc8, 4, 'ZDEMRSA='],
    [0xc8, 5, 'ZDEMRSGI'],
    [0xc9, 1, 'ZE======'],
    [0xc9, 2, 'ZHEQ===='],
    [0xc9, 3, 'ZHE4S==='],
    [0xc9, 4, 'ZHE4TSI='],
    [0xc9, 5, 'ZHE4TSOJ'],
    [0xca, 1, 'ZI======'],
    [0xca, 2, 'ZLFA===='],
    [0xca, 3, 'ZLFMU==='],
    [0xca, 4, 'ZLFMVSQ='],
    [0xca, 5, 'ZLFMVSWK'],
    [0xcb, 1, 'ZM======'],
    [0xcb, 2, 'ZPFQ===='],
    [0xcb, 3, 'ZPF4W==='],
    [0xcb, 4, 'ZPF4XSY='],
    [0xcb, 5, 'ZPF4XS6L'],
    [0xcc, 1, 'ZQ======'],
    [0xcc, 2, 'ZTGA===='],
    [0xcc, 3, 'ZTGMY==='],
    [0xcc, 4, 'ZTGMZTA='],
    [0xcc, 5, 'ZTGMZTGM'],
    [0xcd, 1, 'ZU======'],
    [0xcd, 2, 'ZXGQ===='],
    [0xcd, 3, 'ZXG42==='],
    [0xcd, 4, 'ZXG43TI='],
    [0xcd, 5, 'ZXG43TON'],
    [0xce, 1, 'ZY======'],
    [0xce, 2, 'Z3HA===='],
    [0xce, 3, 'Z3HM4==='],
    [0xce, 4, 'Z3HM5TQ='],
    [0xce, 5, 'Z3HM5TWO'],
    [0xcf, 1, 'Z4======'],
    [0xcf, 2, 'Z7HQ===='],
    [0xcf, 3, 'Z7H46==='],
    [0xcf, 4, 'Z7H47TY='],
    [0xcf, 5, 'Z7H47T6P'],
    [0xd0, 1, '2A======'],
    [0xd0, 2, '2DIA===='],
    [0xd0, 3, '2DINA==='],
    [0xd0, 4, '2DINBUA='],
    [0xd0, 5, '2DINBUGQ'],
    [0xd1, 1, '2E======'],
    [0xd1, 2, '2HIQ===='],
    [0xd1, 3, '2HI5C==='],
    [0xd1, 4, '2HI5DUI='],
    [0xd1, 5, '2HI5DUOR'],
    [0xd2, 1, '2I======'],
    [0xd2, 2, '2LJA===='],
    [0xd2, 3, '2LJNE==='],
    [0xd2, 4, '2LJNFUQ='],
    [0xd2, 5, '2LJNFUWS'],
    [0xd3, 1, '2M======'],
    [0xd3, 2, '2PJQ===='],
    [0xd3, 3, '2PJ5G==='],
    [0xd3, 4, '2PJ5HUY='],
    [0xd3, 5, '2PJ5HU6T'],
    [0xd4, 1, '2Q======'],
    [0xd4, 2, '2TKA===='],
    [0xd4, 3, '2TKNI==='],
    [0xd4, 4, '2TKNJVA='],
    [0xd4, 5, '2TKNJVGU'],
    [0xd5, 1, '2U======'],
    [0xd5, 2, '2XKQ===='],
    [0xd5, 3, '2XK5K==='],
    [0xd5, 4, '2XK5LVI='],
    [0xd5, 5, '2XK5LVOV'],
    [0xd6, 1, '2Y======'],
    [0xd6, 2, '23LA===='],
    [0xd6, 3, '23LNM==='],
    [0xd6, 4, '23LNNVQ='],
    [0xd6, 5, '23LNNVWW'],
    [0xd7, 1, '24======'],
    [0xd7, 2, '27LQ===='],
    [0xd7, 3, '27L5O==='],
    [0xd7, 4, '27L5PVY='],
    [0xd7, 5, '27L5PV6X'],
    [0xd8, 1, '3A======'],
    [0xd8, 2, '3DMA===='],
    [0xd8, 3, '3DMNQ==='],
    [0xd8, 4, '3DMNRWA='],
    [0xd8, 5, '3DMNRWGY'],
    [0xd9, 1, '3E======'],
    [0xd9, 2, '3HMQ===='],
    [0xd9, 3, '3HM5S==='],
    [0xd9, 4, '3HM5TWI='],
    [0xd9, 5, '3HM5TWOZ'],
    [0xda, 1, '3I======'],
    [0xda, 2, '3LNA===='],
    [0xda, 3, '3LNNU==='],
    [0xda, 4, '3LNNVWQ='],
    [0xda, 5, '3LNNVWW2'],
    [0xdb, 1, '3M======'],
    [0xdb, 2, '3PNQ===='],
    [0xdb, 3, '3PN5W==='],
    [0xdb, 4, '3PN5XWY='],
    [0xdb, 5, '3PN5XW63'],
    [0xdc, 1, '3Q======'],
    [0xdc, 2, '3TOA===='],
    [0xdc, 3, '3TONY==='],
    [0xdc, 4, '3TONZXA='],
    [0xdc, 5, '3TONZXG4'],
    [0xdd, 1, '3U======'],
    [0xdd, 2, '3XOQ===='],
    [0xdd, 3, '3XO52==='],
    [0xdd, 4, '3XO53XI='],
    [0xdd, 5, '3XO53XO5'],
    [0xde, 1, '3Y======'],
    [0xde, 2, '33PA===='],
    [0xde, 3, '33PN4==='],
    [0xde, 4, '33PN5XQ='],
    [0xde, 5, '33PN5XW6'],
    [0xdf, 1, '34======'],
    [0xdf, 2, '37PQ===='],
    [0xdf, 3, '37P56==='],
    [0xdf, 4, '37P57XY='],
    [0xdf, 5, '37P57X67'],
    [0xe0, 1, '4A======'],
    [0xe0, 2, '4DQA===='],
    [0xe0, 3, '4DQOA==='],
    [0xe0, 4, '4DQOBYA='],
    [0xe0, 5, '4DQOBYHA'],
    [0xe1, 1, '4E======'],
    [0xe1, 2, '4HQQ===='],
    [0xe1, 3, '4HQ6C==='],
    [0xe1, 4, '4HQ6DYI='],
    [0xe1, 5, '4HQ6DYPB'],
    [0xe2, 1, '4I======'],
    [0xe2, 2, '4LRA===='],
    [0xe2, 3, '4LROE==='],
    [0xe2, 4, '4LROFYQ='],
    [0xe2, 5, '4LROFYXC'],
    [0xe3, 1, '4M======'],
    [0xe3, 2, '4PRQ===='],
    [0xe3, 3, '4PR6G==='],
    [0xe3, 4, '4PR6HYY='],
    [0xe3, 5, '4PR6HY7D'],
    [0xe4, 1, '4Q======'],
    [0xe4, 2, '4TSA===='],
    [0xe4, 3, '4TSOI==='],
    [0xe4, 4, '4TSOJZA='],
    [0xe4, 5, '4TSOJZHE'],
    [0xe5, 1, '4U======'],
    [0xe5, 2, '4XSQ===='],
    [0xe5, 3, '4XS6K==='],
    [0xe5, 4, '4XS6LZI='],
    [0xe5, 5, '4XS6LZPF'],
    [0xe6, 1, '4Y======'],
    [0xe6, 2, '43TA===='],
    [0xe6, 3, '43TOM==='],
    [0xe6, 4, '43TONZQ='],
    [0xe6, 5, '43TONZXG'],
    [0xe7, 1, '44======'],
    [0xe7, 2, '47TQ===='],
    [0xe7, 3, '47T6O==='],
    [0xe7, 4, '47T6PZY='],
    [0xe7, 5, '47T6PZ7H'],
    [0xe8, 1, '5A======'],
    [0xe8, 2, '5DUA===='],
    [0xe8, 3, '5DUOQ==='],
    [0xe8, 4, '5DUOR2A='],
    [0xe8, 5, '5DUOR2HI'],
    [0xe9, 1, '5E======'],
    [0xe9, 2, '5HUQ===='],
    [0xe9, 3, '5HU6S==='],
    [0xe9, 4, '5HU6T2I='],
    [0xe9, 5, '5HU6T2PJ'],
    [0xea, 1, '5I======'],
    [0xea, 2, '5LVA===='],
    [0xea, 3, '5LVOU==='],
    [0xea, 4, '5LVOV2Q='],
    [0xea, 5, '5LVOV2XK'],
    [0xeb, 1, '5M======'],
    [0xeb, 2, '5PVQ===='],
    [0xeb, 3, '5PV6W==='],
    [0xeb, 4, '5PV6X2Y='],
    [0xeb, 5, '5PV6X27L'],
    [0xec, 1, '5Q======'],
    [0xec, 2, '5TWA===='],
    [0xec, 3, '5TWOY==='],
    [0xec, 4, '5TWOZ3A='],
    [0xec, 5, '5TWOZ3HM'],
    [0xed, 1, '5U======'],
    [0xed, 2, '5XWQ===='],
    [0xed, 3, '5XW62==='],
    [0xed, 4, '5XW633I='],
    [0xed, 5, '5XW633PN'],
    [0xee, 1, '5Y======'],
    [0xee, 2, '53XA===='],
    [0xee, 3, '53XO4==='],
    [0xee, 4, '53XO53Q='],
    [0xee, 5, '53XO53XO'],
    [0xef, 1, '54======'],
    [0xef, 2, '57XQ===='],
    [0xef, 3, '57X66==='],
    [0xef, 4, '57X673Y='],
    [0xef, 5, '57X6737P'],
    [0xf0, 1, '6A======'],
    [0xf0, 2, '6DYA===='],
    [0xf0, 3, '6DYPA==='],
    [0xf0, 4, '6DYPB4A='],
    [0xf0, 5, '6DYPB4HQ'],
    [0xf1, 1, '6E======'],
    [0xf1, 2, '6HYQ===='],
    [0xf1, 3, '6HY7C==='],
    [0xf1, 4, '6HY7D4I='],
    [0xf1, 5, '6HY7D4PR'],
    [0xf2, 1, '6I======'],
    [0xf2, 2, '6LZA===='],
    [0xf2, 3, '6LZPE==='],
    [0xf2, 4, '6LZPF4Q='],
    [0xf2, 5, '6LZPF4XS'],
    [0xf3, 1, '6M======'],
    [0xf3, 2, '6PZQ===='],
    [0xf3, 3, '6PZ7G==='],
    [0xf3, 4, '6PZ7H4Y='],
    [0xf3, 5, '6PZ7H47T'],
    [0xf4, 1, '6Q======'],
    [0xf4, 2, '6T2A===='],
    [0xf4, 3, '6T2PI==='],
    [0xf4, 4, '6T2PJ5A='],
    [0xf4, 5, '6T2PJ5HU'],
    [0xf5, 1, '6U======'],
    [0xf5, 2, '6X2Q===='],
    [0xf5, 3, '6X27K==='],
    [0xf5, 4, '6X27L5I='],
    [0xf5, 5, '6X27L5PV'],
    [0xf6, 1, '6Y======'],
    [0xf6, 2, '633A===='],
    [0xf6, 3, '633PM==='],
    [0xf6, 4, '633PN5Q='],
    [0xf6, 5, '633PN5XW'],
    [0xf7, 1, '64======'],
    [0xf7, 2, '673Q===='],
    [0xf7, 3, '6737O==='],
    [0xf7, 4, '6737P5Y='],
    [0xf7, 5, '6737P57X'],
    [0xf8, 1, '7A======'],
    [0xf8, 2, '7D4A===='],
    [0xf8, 3, '7D4PQ==='],
    [0xf8, 4, '7D4PR6A='],
    [0xf8, 5, '7D4PR6HY'],
    [0xf9, 1, '7E======'],
    [0xf9, 2, '7H4Q===='],
    [0xf9, 3, '7H47S==='],
    [0xf9, 4, '7H47T6I='],
    [0xf9, 5, '7H47T6PZ'],
    [0xfa, 1, '7I======'],
    [0xfa, 2, '7L5A===='],
    [0xfa, 3, '7L5PU==='],
    [0xfa, 4, '7L5PV6Q='],
    [0xfa, 5, '7L5PV6X2'],
    [0xfb, 1, '7M======'],
    [0xfb, 2, '7P5Q===='],
    [0xfb, 3, '7P57W==='],
    [0xfb, 4, '7P57X6Y='],
    [0xfb, 5, '7P57X673'],
    [0xfc, 1, '7Q======'],
    [0xfc, 2, '7T6A===='],
    [0xfc, 3, '7T6PY==='],
    [0xfc, 4, '7T6PZ7A='],
    [0xfc, 5, '7T6PZ7H4'],
    [0xfd, 1, '7U======'],
    [0xfd, 2, '7X6Q===='],
    [0xfd, 3, '7X672==='],
    [0xfd, 4, '7X6737I='],
    [0xfd, 5, '7X6737P5'],
    [0xfe, 1, '7Y======'],
    [0xfe, 2, '737A===='],
    [0xfe, 3, '737P4==='],
    [0xfe, 4, '737P57Q='],
    [0xfe, 5, '737P57X6'],
    [0xff, 1, '74======'],
    [0xff, 2, '777Q===='],
    [0xff, 3, '77776==='],
    [0xff, 4, '777777Y='],
    [0xff, 5, '77777777'],
  );

  foreach my $paTuple (@aTuples)
  {
    my $sString = pack("C", $$paTuple[0]) x $$paTuple[1];
    my $sActualValue = EadBase32Encode($sString);
    my $sTargetValue = $$paTuple[2];
    ok($sActualValue eq $sTargetValue);
  }

  ####################################################################
  #
  # Test: Base32 decoding.
  #
  ####################################################################

  foreach my $paTuple (@aTuples)
  {
    my $sHexNumber = sprintf("0x%02x", $$paTuple[0]);
    my $sActualValue = EadBase32Decode($$paTuple[2]);
    my $sTargetValue = chr($$paTuple[0]) x $$paTuple[1];
    ok($sActualValue eq $sTargetValue);
  }

